/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:          D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:               The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/


#include "EQMOM.H"
#include "addToRunTimeSelectionTable.H"
#include "fvc.H"
#include "zeroGradientFvPatchFields.H"
#include "uniformDimensionedFields.H"
#include "fixedValueFvPatchFields.H"    
#include "wallFvPatch.H"


namespace Foam
{
    defineTypeNameAndDebug(EQMOM, 0);

    addToRunTimeSelectionTable
    (
        MOM,
        EQMOM,
        dictionary
    );
}


void Foam::EQMOM::calculateAbs
(
    const label& cellI
)
{
    dummyAbs_[cellI] = 
        log
        (
            sqr(M_[1][cellI])/M_[0][cellI]
           /sqrt(M_[2][cellI]*M_[0][cellI])
        );

	sigma_ = sqrt(2.0*log(sqrt(M_[0][cellI]*M_[2][cellI])/M_[1][cellI]));

    scalar dominator = 1.0/sqrt(M_PI);

    forAll(abs_, kth)
    {
        abs_[kth][cellI] = exp(sqrt(2.0)*sigma_[cellI]*abscissasSecond_[kth] + dummyAbs_[cellI]);

        weis_[kth][cellI] = M_[0][cellI]*weightsSecond_[kth]*dominator;
    }

    d32_.ref()[cellI] = M_[3][cellI]/M_[2][cellI];
}


Foam::EQMOM::EQMOM
(
    const fvMesh& mesh,
    const dictionary& MOMProperties
)
:
    MOM(mesh, MOMProperties),
    dummyAbs_
    (
        IOobject
        (
            "dummyAbs",
            mesh_.time().timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh_,
        dimensionedScalar
        (
            "dummyAbs",
            dimLength,
            SMALL
        )
    ),
    sigma_
	(
		IOobject
        (
            "sigma",
            mesh_.time().timeName(),
			mesh_,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh_,
		scalar(0.0),
		zeroGradientFvPatchField<scalar>::typeName
	)
{
    Info<<"\nEQMOM constructing function\n";
   
    //weightsSecond_[0]=0.8862269254527580136491;
	//weightsSecond_[1]=0.8862269254527580136491;
	//abscissasSecond_[0]=-0.7071067811865475244008;
	//abscissasSecond_[1]=0.7071067811865475244008;

    weightsSecond_[0]=0.081312835447245177143;
	weightsSecond_[1]=0.8049140900055128365061;
	weightsSecond_[2]=0.8049140900055128365061;
	weightsSecond_[3]=0.081312835447245177143;
	abscissasSecond_[0]=-1.650680123885784555883;
	abscissasSecond_[1]=-0.5246476232752903178841;
	abscissasSecond_[2]=0.5246476232752903178841;
	abscissasSecond_[3]=1.650680123885784555883;

    //weightsSecond_[0] = 2.908254700131226229411E-21;
    //weightsSecond_[1] = 2.8103336027509037088E-17;
    //weightsSecond_[2] = 2.87860708054870606219E-14;
    //weightsSecond_[3] = 8.106186297463044204E-12;
    //weightsSecond_[4] = 9.1785804243785282085E-10;
    //weightsSecond_[5] = 5.10852245077594627739E-8;
    //weightsSecond_[6] = 1.57909488732471028835E-6;
    //weightsSecond_[7] = 2.9387252289229876415E-5;
    //weightsSecond_[8] = 3.48310124318685523421E-4;
    //weightsSecond_[9] = 0.00273792247306765846299;
    //weightsSecond_[10] = 0.0147038297048266835153;
    //weightsSecond_[11] = 0.0551441768702342511681;
    //weightsSecond_[12] = 0.1467358475408900997517;
    //weightsSecond_[13] = 0.2801309308392126674135;
    //weightsSecond_[14] = 0.386394889541813862556;
    //weightsSecond_[15] = 0.3863948895418138625556;
    //weightsSecond_[16] = 0.2801309308392126674135;
    //weightsSecond_[17] = 0.1467358475408900997517;
    //weightsSecond_[18] = 0.0551441768702342511681;
    //weightsSecond_[19] = 0.01470382970482668351528;
    //weightsSecond_[20] = 0.002737922473067658462989;
    //weightsSecond_[21] = 3.48310124318685523421E-4;
    //weightsSecond_[22] = 2.938725228922987641501E-5;
    //weightsSecond_[23] = 1.579094887324710288346E-6;
    //weightsSecond_[24] = 5.1085224507759462774E-8;
    //weightsSecond_[25] = 9.1785804243785282085E-10;
    //weightsSecond_[26] = 8.10618629746304420399E-12;
    //weightsSecond_[27] = 2.87860708054870606219E-14;
    //weightsSecond_[28] = 2.81033360275090370876E-17;
    //weightsSecond_[29] = 2.9082547001312262294E-21;
    //abscissasSecond_[0] = -6.863345293529891581061;
    //abscissasSecond_[1] = -6.138279220123934620395;
    //abscissasSecond_[2] = -5.533147151567495725118;
    //abscissasSecond_[3] = -4.988918968589943944486;
    //abscissasSecond_[4] = -4.48305535709251834189;
    //abscissasSecond_[5] = -4.003908603861228815228;
    //abscissasSecond_[6] = -3.544443873155349886925;
    //abscissasSecond_[7] = -3.099970529586441748689;
    //abscissasSecond_[8] = -2.667132124535617200571;
    //abscissasSecond_[9] = -2.243391467761504072473;
    //abscissasSecond_[10] = -1.826741143603688038836;
    //abscissasSecond_[11] = -1.415527800198188511941;
    //abscissasSecond_[12] = -1.008338271046723461805;
    //abscissasSecond_[13] = -0.6039210586255523077782;
    //abscissasSecond_[14] = -0.2011285765488714855458;
    //abscissasSecond_[15] = 0.2011285765488714855458;
    //abscissasSecond_[16] = 0.6039210586255523077782;
    //abscissasSecond_[17] = 1.008338271046723461805;
    //abscissasSecond_[18] = 1.415527800198188511941;
    //abscissasSecond_[19] = 1.826741143603688038836;
    //abscissasSecond_[20] = 2.243391467761504072473;
    //abscissasSecond_[21] = 2.667132124535617200571;
    //abscissasSecond_[22] = 3.099970529586441748689;
    //abscissasSecond_[23] = 3.544443873155349886925;
    //abscissasSecond_[24] = 4.003908603861228815228;
    //abscissasSecond_[25] = 4.48305535709251834189;
    //abscissasSecond_[26] = 4.988918968589943944486;
    //abscissasSecond_[27] = 5.533147151567495725118;
    //abscissasSecond_[28] = 6.138279220123934620395;
    //abscissasSecond_[29] = 6.863345293529891581061;

    //weightsSecond_[0]=0.000007640432855232646075;
	//weightsSecond_[1]=0.0013436457467812332883;
	//weightsSecond_[2]=0.033874394455481043886;
	//weightsSecond_[3]=0.24013861108231340791;
	//weightsSecond_[4]=0.61086263373532512233;
	//weightsSecond_[5]=0.61086263373532512233;
	//weightsSecond_[6]=0.24013861108231340791;
	//weightsSecond_[7]=0.033874394455481043886;
	//weightsSecond_[8]=0.0013436457467812332883;
	//weightsSecond_[9]=0.000007640432855232646075;
	//abscissasSecond_[0]=3.4361591188377378359;
	//abscissasSecond_[1]=2.5327316742327896648;
	//abscissasSecond_[2]=1.7566836492998818553;
	//abscissasSecond_[3]=1.036610829789513577;
	//abscissasSecond_[4]=0.3429013272237045884;
	//abscissasSecond_[5]=-0.3429013272237045884;
	//abscissasSecond_[6]=-1.036610829789513577;
	//abscissasSecond_[7]=-1.7566836492998818553;
	//abscissasSecond_[8]=-2.5327316742327896648;
	//abscissasSecond_[9]=-3.4361591188377378359;
}


