/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:          D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:               The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "MOM.H"
#include "fvm.H"
#include "fixedValueFvPatchFields.H"    
#include "zeroGradientFvPatchFields.H"
#include "parcel.H"


namespace Foam
{
    defineTypeNameAndDebug(MOM, 0);
    defineRunTimeSelectionTable(MOM, dictionary);
}


void Foam::MOM::solve()
{
    /* * * * * * * * * * * * * Moments advection step * * * * * * * * * * * * */
    
    forAll(M_[0], cellI)
    {
        //- d32 updated inside
        calculateAbs(cellI);
    
        updateVPC(cellI);
    
        updateVelocity(cellI);
    }

    //- Correct boundary for the abscissas, weights and velocities
    correctBoundaryAndCells();
    
    realizableMom_.correctFlux
    (
        weis_,
        abs_,
        M_,
        mFluxX_,
        mFluxY_,
        mFluxZ_,
        particleU_,
        pureMomentsNumber_,
        nodes_
    );
    
    Info<< "Solving for moments advection";
    forAll(M_, kth)
    {
        scalarField& psiIf = M_[kth];
        const scalarField& psi0 = M_[kth].oldTime();
        const scalar deltaT = mesh_.time().deltaT().value();

        psiIf = 0.0;
        surfaceScalarField phiPsi = mFluxX_[kth] + mFluxY_[kth] + mFluxZ_[kth];
        
        fvc::surfaceIntegrate(psiIf, phiPsi);

        psiIf = (psi0/deltaT - psiIf)*deltaT;

        M_[kth].correctBoundaryConditions();
    }
    Ud_.correctBoundaryConditions();
    
    Info<< ", End moments advection " << nl
        << "Solving for moments force updating";

    /* * * * * * * * * * * * * Force computing step * * * * * * * * * * * * * */

    if (forceTerm_)
    {
        updateMomentsForce();
    }

    Info<< ", End moments force updating\n";
}


void Foam::MOM::updateVPC
(
    const label& cellI
)
{
    scalar matrixCoeff;

    if (nodes_ < 4)
    {
        matrixCoeff = nodes_;
    }
    else
    {
        matrixCoeff = 3;
    }
    
    scalarDiagonalMatrix rhsU(matrixCoeff, 0);
    scalarDiagonalMatrix rhsV(matrixCoeff, 0);
    scalarDiagonalMatrix rhsW(matrixCoeff, 0);
    
    //- U component
    {
        SquareMatrix<scalar>  momentsMatrix(matrixCoeff);
    
        for (int i = 0; i < matrixCoeff; i++)
        {
            for (int j = 0; j < matrixCoeff; j++)
            {
                momentsMatrix(i, j) = M_[i + j][cellI];
            }
        
            rhsU[i] = M_[i + pureMomentsNumber_][cellI];
        }
        
        LUsolve(momentsMatrix, rhsU);
    }
    
    //- V component
    {
        SquareMatrix<scalar>  momentsMatrix(matrixCoeff);
    
        for (int i = 0; i < matrixCoeff; i++)
        {
            for (int j = 0; j < matrixCoeff; j++)
            {
                momentsMatrix(i, j) = M_[i + j][cellI];
            }
            rhsV[i] = M_[i + pureMomentsNumber_ + nodes_][cellI];
        }
    
        LUsolve(momentsMatrix, rhsV);
    }

    //- W component
    {
        SquareMatrix<scalar>  momentsMatrix(matrixCoeff);
    
        for (int i = 0; i < matrixCoeff; i++)
        {
            for (int j = 0; j < matrixCoeff; j++)
            {
                momentsMatrix(i, j) = M_[i + j][cellI];
            }
            rhsW[i] = M_[i + pureMomentsNumber_ + nodes_*2][cellI];
        }
        
        LUsolve(momentsMatrix, rhsW);
    }

    if (nodes_ > 2)
    {
        for (int i = 0; i < 3; i++)
        {
            VPCU_[cellI].component(i) = rhsU[i];
            VPCV_[cellI].component(i) = rhsV[i];
            VPCW_[cellI].component(i) = rhsW[i];
        }
    }
    else if (nodes_ == 2)
    {
        for (int i = 0; i < 2; i++)
        {
            VPCU_[cellI].component(i) = rhsU[i];
            VPCV_[cellI].component(i) = rhsV[i];
            VPCW_[cellI].component(i) = rhsW[i];
        }
        VPCU_[cellI].component(3) = 0;
        VPCV_[cellI].component(3) = 0;
        VPCW_[cellI].component(3) = 0;
    }
    else if (nodes_ == 1)
    {
        VPCU_[cellI].component(0) = rhsU[0];
        VPCV_[cellI].component(0) = rhsV[0];
        VPCW_[cellI].component(0) = rhsW[0];

        for (int i = 2; i < 4; i++)
        {
            VPCU_[cellI].component(i) = 0;
            VPCV_[cellI].component(i) = 0;
            VPCW_[cellI].component(i) = 0;
        }
    }
}


void Foam::MOM::mixedMomentsUpdate(const label& cellI)
{
    forAll(M_, kth)
    {
        if 
        (
            kth > pureMomentsNumber_ - 1
         && kth < pureMomentsNumber_ + nodes_
        )
        {
            M_[kth][cellI] = 0.0;
            for (int n = 0; n < abs_.size(); n++)
            {
                M_[kth][cellI] += 
                    weis_[n][cellI]*particleU_[n][cellI].x()
                   *Foam::pow(abs_[n][cellI], kth - pureMomentsNumber_);
            }
        }

        if 
        (
            kth > pureMomentsNumber_ - 1 + nodes_
         && kth < pureMomentsNumber_ + 2*nodes_
        )
        {
            M_[kth][cellI] = 0.0;
            for (int n = 0; n < abs_.size(); n++)
            {
                M_[kth][cellI] +=
                    weis_[n][cellI]*particleU_[n][cellI].y()
                   *Foam::pow
                    (
                        abs_[n][cellI], kth - pureMomentsNumber_ - nodes_
                    );
            }
        }

        if 
        (
            kth > pureMomentsNumber_ - 1 + 2*nodes_
         && kth < pureMomentsNumber_ + 3*nodes_
        )
        {
            M_[kth][cellI] = 0.0;
            for (int n = 0; n < abs_.size(); n++)
            {
                M_[kth][cellI] +=
                    weis_[n][cellI]*particleU_[n][cellI].z()
                   *Foam::pow
                    (
                        abs_[n][cellI], kth - pureMomentsNumber_ - 2*nodes_
                    );
            }
        }
    }
}


void Foam::MOM::updateMomentsForce()
{
    updatebubblePressureForce();

    const volVectorField& Uc =  
        mesh_.lookupObject<volVectorField>("U.water");

    const volScalarField& muc = 
        mesh_.lookupObject<volScalarField>("mu.water");

    const volScalarField& rhoc = 
        mesh_.lookupObject<volScalarField>("rho.water");

    const volScalarField& k = 
        mesh_.lookupObject<volScalarField>("k");

    const volScalarField& nut = 
        mesh_.lookupObject<volScalarField>("nut");

    const scalar& dT = mesh_.time().deltaTValue();

    const uniformDimensionedVectorField& g = 
        mesh_.lookupObject<uniformDimensionedVectorField>("g");

    volVectorField curlUc("curlUc", fvc::curl(Uc));

    forAll(M_[0], cellI)
    {
        const int& n = numberOfAbs_;

        vector* gradPhaseFrac = new vector[n];
        vector* bPGrad = new vector[n];
        
        for (int kth = 0; kth < numberOfAbs_; kth++)
        {
            gradPhaseFrac[kth] = gradPhaseFraction_[kth]()[cellI];
            bPGrad[kth] = bubblePressureGrad_[kth][cellI];
        }

        //- (1)
        calculateAbs(cellI);

        //- (2)
        updateVPC(cellI);

        //- (3)
        updateVelocity(cellI);

        //- (4)
        velocityODE(cellI, Uc[cellI], muc[cellI], rhoc[cellI], dT, g, 
            curlUc[cellI], gradPhaseFrac, bPGrad, k[cellI], nut[cellI]);

        //- (5) 
        mixedMomentsUpdate(cellI);

        delete [] gradPhaseFrac;
        delete [] bPGrad;
    }

    //- Correct boundary after force term
    correctBoundaryAndCells();
}


void Foam::MOM::velocityODE
(
    const label& cellI,
    const vector& Uc,
    const scalar& muc,
    const scalar& rhoc,
    const scalar& dT,
    const uniformDimensionedVectorField& g,
    const vector& curlUc,
    const vector* gradPhaseFrac,
    const vector* bPGrad,
    const scalar& k,
    const scalar& nut
)
{
    forAll(abs_, kth)
    {
        vector& Ud(particleU_[kth].primitiveFieldRef()[cellI]);

        const scalar& phaseF(alphaSequence_[kth][cellI]);

        parcel p_(cellI, Ud, Uc, curlUc, k, rhod_.value(), rhoc, muc, nut,
            abs_[kth][cellI], phaseF, alpha_[cellI], gradPhaseFrac, kth);
    
        //- Update values
        forceFrame_.updateCoefficients(p_);
    
        //- Initialisation 
        const scalar& Kd = forceFrame_.Kd()[kth][cellI]/phaseF;
    
        const scalar& Sp = Kd/rhod_.value();
    
        const vector& wallLubF = forceFrame_.wallLubF()[kth][cellI];

        const vector& liftF = forceFrame_.liftF()[kth][cellI];
    
        const vector& turbulentDisF = forceFrame_.turbulentDisF()[kth][cellI];
    
        SU_[kth][cellI] = g.value()*(1.0 - rhoc/rhod_.value())
          + liftF/phaseF*weis_[kth][cellI]/M_[0][cellI]
          + wallLubF/phaseF*weis_[kth][cellI]/M_[0][cellI]
          + bPGrad[kth]/phaseF
          + turbulentDisF/phaseF;

        timeSchemePtr_->ODEresult(dT, phaseF, Uc, Sp, SU_[kth][cellI], Ud);
    }
}


void Foam::MOM::updatebubblePressureForce()
{
    const volVectorField& Uc =  
        mesh_.lookupObject<volVectorField>("U.water");

    const volScalarField& rhoc = 
        mesh_.lookupObject<volScalarField>("rho.water");

    for (int kth = 0; kth < numberOfAbs_; kth++)
    {
        gradPhaseFraction_[kth].ref() = fvc::grad(alphaSequence_[kth]);

        bubblePressureGrad_[kth].ref() = 
         - alphaSequence_[kth]*rhoc
         * fvc::grad
           (
               BP_*alphaSequence_[kth]*magSqr(particleU_[kth] - Uc),
              //*(1.0 - alphaSequence_[kth]), 
               "grad(bubblePressure)"
           );
    }
}


void Foam::MOM::correctVPCboundary()
{
    scalar matrixCoeff;

    if (nodes_ < 4)
    {
        matrixCoeff = nodes_;
    }
    else
    {
        matrixCoeff = 3;
    }

    scalarDiagonalMatrix rhsU(matrixCoeff, 0);
    scalarDiagonalMatrix rhsV(matrixCoeff, 0);
    scalarDiagonalMatrix rhsW(matrixCoeff, 0);

    const fvPatchList& patches = mesh_.boundary();
    
    forAll(M_[0].boundaryField(), p)
    {
        if 
        (
          ! isA<zeroGradientFvPatchScalarField>
            (
                M_[0].boundaryField()[p]
            )
          &&
          ! isA<fixedValueFvPatchScalarField>
            (
                M_[0].boundaryField()[p]
            )
        )
        {
            forAll(patches[p], f)
            {
                //- U component
                {
                    SquareMatrix<scalar>  momentsMatrix(matrixCoeff);
        
                    for (int i = 0; i < matrixCoeff; i++)
                    {
                        for (int j = 0; j < matrixCoeff; j++)
                        {
                            momentsMatrix(i, j) 
                              = M_[i + j].boundaryField()[p][f];
                        }
                    
                        rhsU[i] = 
                            M_[i + pureMomentsNumber_].boundaryField()[p][f];
                    }
                    
                    LUsolve(momentsMatrix, rhsU);
                }

                //- V component
                {
                    SquareMatrix<scalar>  momentsMatrix(matrixCoeff);
                    
                    for (int i = 0; i < matrixCoeff; i++)
                    {
                        for (int j = 0; j < matrixCoeff; j++)
                        {
                            momentsMatrix(i, j) 
                              = M_[i + j].boundaryField()[p][f];
                        }
                        rhsV[i] = 
                            M_
                            [
                                i + pureMomentsNumber_ + nodes_
                            ].boundaryField()[p][f];
                    }
                    
                    LUsolve(momentsMatrix, rhsV);
                }

                //- W component
                {
                    SquareMatrix<scalar>  momentsMatrix(matrixCoeff);
            
                    for (int i = 0; i < matrixCoeff; i++)
                    {
                        for (int j = 0; j < matrixCoeff; j++)
                        {
                            momentsMatrix(i, j) 
                              = M_[i + j].boundaryField()[p][f];
                        }
                        rhsW[i] = 
                            M_
                            [
                                i + pureMomentsNumber_ + nodes_*2
                            ].boundaryField()[p][f];
                    }
                    
                    LUsolve(momentsMatrix, rhsW);
                }

                if (nodes_ > 2)
                {
                    for (int i = 0; i < 3; i++)
                    {
                        VPCU_.boundaryFieldRef()[p][f].component(i) = rhsU[i];
                        VPCV_.boundaryFieldRef()[p][f].component(i) = rhsV[i];
                        VPCW_.boundaryFieldRef()[p][f].component(i) = rhsW[i];
                    }
                }
                else if (nodes_ == 2)
                {
                    for (int i = 0; i < 2; i++)
                    {
                        VPCU_.boundaryFieldRef()[p][f].component(i) = rhsU[i];
                        VPCV_.boundaryFieldRef()[p][f].component(i) = rhsV[i];
                        VPCW_.boundaryFieldRef()[p][f].component(i) = rhsW[i];
                    }
                    VPCU_.boundaryFieldRef()[p][f].component(3) = 0;
                    VPCV_.boundaryFieldRef()[p][f].component(3) = 0;
                    VPCW_.boundaryFieldRef()[p][f].component(3) = 0;
               }
               else if (nodes_ == 1)
               {
                   VPCU_.boundaryFieldRef()[p][f].component(0) = rhsU[0];
                   VPCV_.boundaryFieldRef()[p][f].component(0) = rhsV[0];
                   VPCW_.boundaryFieldRef()[p][f].component(0) = rhsW[0];

                   for (int i = 2; i < 4; i++)
                   {
                       VPCU_.boundaryFieldRef()[p][f].component(i) = 0;
                       VPCV_.boundaryFieldRef()[p][f].component(i) = 0;
                       VPCW_.boundaryFieldRef()[p][f].component(i) = 0;
                   }
               }
            }
        }
    }
}


void Foam::MOM::solve0D()
{
    forAll(M_[0], cellI)
    {
        calculateAbs(cellI);
    }
}


void Foam::MOM::RK2SSP()
{
    PtrList<tmp<volScalarField>> Mzero(15);
    PtrList<tmp<volScalarField>> Mzerozero(15);

    forAll(Mzero, kth)
    {
        Mzero.set
        (
            kth,
            new tmp<volScalarField> 
            (
                new volScalarField
                (  
                    IOobject
                    (
                        "Mzero" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    M_[kth]
                )
            )
        );
    }

    for (int RKloop = 0; RKloop < 3; RKloop++)
    {
        solve();

        if (RKloop == 1)
        {
            forAll(Mzero, kth)
            {
                Mzerozero.set
                (
                    kth,
                    new tmp<volScalarField> 
                    (
                        new  volScalarField
                        (  
                            IOobject
                            (
                                "Mzerozero" + name(kth),
                                mesh_.time().timeName(),
                                mesh_,
                                IOobject::NO_READ,
                                IOobject::NO_WRITE
                            ),
                            M_[kth]
                        )
                    )
                );
            }
        }
    }

    forAll(Mzero, kth)
    {
        M_[kth] = 1.0/3.0*(M_[kth] + Mzero[kth]() + Mzerozero[kth]());
    }
}


void Foam::MOM::correctBoundaryAndCells()
{
    //- 
    correctVPCboundary();

    //- Correct weights and abs
    momInvPtr_->solveBoundary
    (
        weis_,
        abs_,
        M_,
        numberOfAbs_
    );

    //- Correct particleU boundary
    const fvPatchList& patches = mesh_.boundary();

    forAll(particleU_, kth)
    {
        //- Update cell values
        alphaSequence_[kth] = M_PI/6.0*weis_[kth]*pow3(abs_[kth]);

        forAll(patches, p)
        {
            const fvPatch& currPatch = patches[p];

            //- Fixed value
            if 
            (
                isA<fixedValueFvPatchScalarField>
                (
                    M_[3].boundaryField()[p]
                )
            )
            {
                forAll(patches[p], f)
                {
                    VPCU_.boundaryFieldRef()[p][f] = 
                        VPCV_.boundaryFieldRef()[p][f] = 
                        VPCW_.boundaryFieldRef()[p][f] = 
                        Zero;
                    
                    particleU_[kth].boundaryFieldRef()[p][f].x() = 
                        Ud_.boundaryField()[p][f].x();

                    particleU_[kth].boundaryFieldRef()[p][f].y() = 
                        Ud_.boundaryField()[p][f].y();

                    particleU_[kth].boundaryFieldRef()[p][f].z() = 
                        Ud_.boundaryField()[p][f].z();
                    
                    alphaSequence_[kth].boundaryFieldRef()[p][f] = 
                            M_PI/6.0*weis_[kth].boundaryField()[p][f]
                           *pow3(abs_[kth].boundaryField()[p][f]);

                    alpha_.boundaryFieldRef()[p][f] = 0.0;

                    for (int i = 0; i < nodes_; i++)
                    {
                        alpha_.boundaryFieldRef()[p][f] += 
                            alphaSequence_[i].boundaryFieldRef()[p][f];
                    }

                    beta_.boundaryFieldRef()[p][f] = 1.0 
                      - alpha_.boundaryFieldRef()[p][f];
                }
            }
            //- Zero gradient boundary
            else if
            (
                isA<zeroGradientFvPatchScalarField>
                (
                    M_[0].boundaryField()[p]
                )
            )
            {
                forAll(patches[p], f)
                {
                    label faceCelli = currPatch.faceCells()[f];

                    particleU_[kth].boundaryFieldRef()[p][f] = 
                            particleU_[kth][faceCelli];

                    VPCU_.boundaryFieldRef()[p][f] = 
                            VPCU_[faceCelli];

                    VPCV_.boundaryFieldRef()[p][f] = 
                            VPCV_[faceCelli];

                    VPCW_.boundaryFieldRef()[p][f] = 
                            VPCW_[faceCelli];

                    alphaSequence_[kth].boundaryFieldRef()[p][f] = 
                            M_PI/6.0*weis_[kth].boundaryField()[p][f]
                           *pow3(abs_[kth].boundaryField()[p][f]);

                    alpha_.boundaryFieldRef()[p][f] = 0.0;

                    for (int i = 0; i < nodes_; i++)
                    {
                        alpha_.boundaryFieldRef()[p][f] += 
                            alphaSequence_[i].boundaryFieldRef()[p][f];
                    }

                    beta_.boundaryFieldRef()[p][f] = 1.0 
                      - alpha_.boundaryFieldRef()[p][f];
                }
            }
            //- Processor boundary
            else
            {
                forAll(patches[p], f)
                {
                    const scalar& absPsi = abs_[kth].boundaryField()[p][f];

                    particleU_[kth].boundaryFieldRef()[p][f].x() = 
                        VPCU_.boundaryField()[p][f].x()
                      + absPsi*VPCU_.boundaryField()[p][f].y()
                      + sqr(absPsi)*VPCU_.boundaryField()[p][f].z();

                    particleU_[kth].boundaryFieldRef()[p][f].y() = 
                        VPCV_.boundaryField()[p][f].x()
                      + absPsi*VPCV_.boundaryField()[p][f].y()
                      + sqr(absPsi)*VPCV_.boundaryField()[p][f].z();

                    particleU_[kth].boundaryFieldRef()[p][f].z() = 
                        VPCW_.boundaryField()[p][f].x()
                      + absPsi*VPCW_.boundaryField()[p][f].y()
                      + sqr(absPsi)*VPCW_.boundaryField()[p][f].z();

                    alpha_.boundaryFieldRef()[p][f] = 0.0;

                    for (int i = 0; i < nodes_; i++)
                    {
                        alpha_.boundaryFieldRef()[p][f] += 
                            alphaSequence_[i].boundaryFieldRef()[p][f];
                    }

                    beta_.boundaryFieldRef()[p][f] = 1.0 
                      - alpha_.boundaryFieldRef()[p][f];
                }            
            }
        }
    }

    alpha_ = 0.0;

    for (int kth = 0; kth < nodes_; kth++)
    {
        alpha_ += alphaSequence_[kth];
    }

    beta_ = 1.0 - alpha_;
}


void Foam::MOM::updateVelocity
(
    const label& cellI
)
{   
    forAll(particleU_, kth)
    {
        const scalar& absPsi = abs_[kth][cellI];
        particleU_[kth][cellI].x() = 
            VPCU_[cellI].x() + absPsi*VPCU_[cellI].y()
          + sqr(absPsi)*VPCU_[cellI].z();

        particleU_[kth][cellI].y() = 
            VPCV_[cellI].x() + absPsi*VPCV_[cellI].y()
          + sqr(absPsi)*VPCV_[cellI].z();

        particleU_[kth][cellI].z() = 
            VPCW_[cellI].x() + absPsi*VPCW_[cellI].y()
          + sqr(absPsi)*VPCW_[cellI].z();
    }   
}


Foam::MOM::MOM
(
    const fvMesh& mesh,
    const dictionary& MOMProperties
)
:
    mesh_(mesh),
    forceFrame_(MOMProperties, mesh),
    MOMDictionary_(MOMProperties),
    momModel_(MOMProperties.lookup("MOMModel")),
    forceTerm_(MOMProperties.lookup("forceTerm")),
    //- pureMomentsNumber is used to control QMOM or EQMOM
    pureMomentsNumber_(readScalar(MOMDictionary_.lookup("pureMomentsNumber"))),
    numberOfAbs_(readScalar(MOMDictionary_.lookup("nodesNumber"))),
    nodes_(0),
    d32_
    (
        new volScalarField
        (
            IOobject
            (
                "d32",
                mesh_.time().timeName(),
                mesh_,
                IOobject::NO_READ,
                IOobject::AUTO_WRITE
            ),
            mesh_,
            dimensionedScalar
            (
                "d32", 
                dimensionSet(0,1,0,0,0,0,0), 
                1e-6    
            ),
            zeroGradientFvPatchField<scalar>::typeName
        )
    ),
    // 3 donotes three dimensional
    M_(pureMomentsNumber_ + 3*numberOfAbs_),
    //M_(10),//TODO for EQMOM, 10
    alpha_
    (
        IOobject
        (
            "alpha",
            mesh_.time().timeName(),
            mesh_,
            IOobject::READ_IF_PRESENT,
            IOobject::AUTO_WRITE
        ),
        mesh_,
        dimensionedScalar
        (
            "alpha",
            dimless,
            0.0
        )
    ),
    beta_
    (
        IOobject
        (
            "alpha.water",
            mesh_.time().timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        1.0 - alpha_
    ),
    VPCU_
    (
        IOobject
        (
            "VPCU",
            mesh_.time().timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh_,
        vector(0,0,0)
    ),
    VPCV_
    (
        IOobject
        (
            "VPCV",
            mesh_.time().timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh_,
        vector(0, 0, 0)
    ),
    VPCW_
    (
        IOobject
        (
            "VPCW",
            mesh_.time().timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh_,
        vector(0, 0, 0)
    ),
    rhod_
    (
        "rhod",
        dimDensity, 
        MOMProperties.lookup("rhod")
    ),
    rhoc_(VPCV_.db().lookupObject<volScalarField>("rho.water")),
    mFluxX_(M_.size()),
    mFluxY_(M_.size()),
    mFluxZ_(M_.size()),
    Ud_
    (
        IOobject
        (
            "U.air",
            mesh_.time().timeName(),
            mesh_,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE,
            "false"
        ),
        mesh_
    ),
    weis_(numberOfAbs_),
    abs_(numberOfAbs_),
    alphaSequence_(numberOfAbs_),
    bubblePressureGrad_(numberOfAbs_),
    gradPhaseFraction_(numberOfAbs_),
    particleU_(numberOfAbs_),
    momInvPtr_
    (
        momentsInversion::New
        (
            MOMProperties.lookup("momentsInversionModel"),
            mesh_
        )
    ),
    //breakagePtr_(nullptr),
    //coalescencePtr_(nullptr),
    //DSDPtr_(nullptr),
    timeSchemePtr_
    (
        timeScheme::New
        (
            MOMDictionary_
        )
    ),
    realizableMom_
    (
        mesh_,
        M_.size(),
        numberOfAbs_
    ),
    SU_(numberOfAbs_),
    BP_
    (
        "BP",
        dimless,
        MOMDictionary_.lookup("BPCoefficient")
    )
{
    if (momModel_ == "QMOM")
    {
        nodes_ = numberOfAbs_;
    }
    else
    {
        //- EQMOM 
        nodes_ = 2;
    }

    forAll(M_, kth)
    {
        //- Pure moments
        if (kth < pureMomentsNumber_)
        {
            M_.set
            (
                kth,
                new volScalarField 
                (
                    IOobject
                    (
                        "M" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::MUST_READ,
                        IOobject::AUTO_WRITE
                    ),
                    mesh_
                )
            );
            Info<< "M" << kth << M_[kth].dimensions() << nl; 
        }

        //- Mixed moments
        if 
        (
            kth > pureMomentsNumber_ - 1
         && kth < pureMomentsNumber_ + nodes_
        )
        // 6 7 8
        // 4 5
        {
            M_.set
            (
                kth,
                new volScalarField 
                (
                    IOobject
                    (
                        "M" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    Ud_.component(0)*M_[kth - pureMomentsNumber_],
                    M_[kth - pureMomentsNumber_].boundaryField().types()
                )
            );
            Info<< "M" << kth << M_[kth].dimensions() << nl; 
        }

        if 
        (
            kth > pureMomentsNumber_ - 1 + nodes_
         && kth < pureMomentsNumber_ + 2*nodes_
        )
        // 9 10 11
        // 6 7
        {
            M_.set
            (
                kth,
                new volScalarField 
                (
                    IOobject
                    (
                        "M" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    Ud_.component(1)*M_[kth - pureMomentsNumber_ - nodes_],
                    M_[kth - pureMomentsNumber_ - nodes_].boundaryField().types()
                )
            );
            Info<< "M" << kth << M_[kth].dimensions() << nl; 
        }

        if 
        (
            kth > pureMomentsNumber_ - 1 + 2*nodes_
         && kth < pureMomentsNumber_ + 3*nodes_
        )
        // 12 13 14
        // 8 9
        {
            M_.set
            (
                kth,
                new volScalarField 
                (
                    IOobject
                    (
                        "M" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    Ud_.component(2)*M_[kth - pureMomentsNumber_ - 2*nodes_],
                    M_[kth - pureMomentsNumber_ - 2*nodes_].boundaryField().types()
                )
            );
            Info<< "M" << kth << M_[kth].dimensions() << nl; 
        }

        //- surface B.C. can not be zeroGradient
        wordList mFluxTypes
        (
            M_[kth].boundaryField().size(),
            calculatedFvPatchScalarField::typeName
        );
        
        forAll(M_[kth].boundaryField(), i)
        {
            if
            (
                isA<fixedValueFvPatchScalarField>
                (
                    M_[kth].boundaryField()[i]
                )
            )
            {
                mFluxTypes[i] = fixedValueFvPatchScalarField::typeName;
            }
        }
        
        mFluxX_.set
        (
            kth,
            new surfaceScalarField 
            (
                IOobject
                (
                    "mFluxX" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh_,
                dimensionedScalar
                (
                   "mFluxX" + name(kth),
                   M_[kth].dimensions()*dimVolume/dimTime,
                   0.0
                ),
                mFluxTypes
            )
        );
        
        mFluxY_.set
        (
            kth,
            new surfaceScalarField 
            (
                IOobject
                (
                    "mFluxY" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh_,
                dimensionedScalar
                (
                    "mFluxY" + name(kth),
                    M_[kth].dimensions()*dimVolume/dimTime,
                    0.0
                ),
                mFluxTypes
            )
        );
        
        mFluxZ_.set
        (
            kth,
            new surfaceScalarField 
            (
                IOobject
                (
                    "mFluxZ" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh_,
                dimensionedScalar
                (
                    "mFluxZ" + name(kth),
                    M_[kth].dimensions()*dimVolume/dimTime,
                    0.0
                ),
                mFluxTypes
            )
        );
        
        Info<< "MFlux" << kth << mFluxX_[kth].dimensions() << nl; 
    }

    Info<<"\nEnd MOM constructing function\n";
    
    //- Abscissas and weights initialization

    Info<<"\nQMOM constructing function\n";

    forAll(abs_, kth)
    {
        abs_.set
        (
            kth,
            new volScalarField 
            (
                IOobject
                (
                    "abs" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh_,
                dimensionedScalar
                (
                    "abs" + name(kth), 
                    dimensionSet(0,1,0,0,0,0,0), 
                    SMALL
                )
            )
        );
    
        weis_.set
        (
            kth,
            new volScalarField 
            (
                IOobject
                (
                    "weis" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh_,
                dimensionedScalar
                (
                    "weis" + name(kth), 
                    dimensionSet(0,-3,0,0,0,0,0), 
                    SMALL
                )
            )
        );
    
        alphaSequence_.set
        (
            kth,
            new volScalarField 
            (
                IOobject
                (
                    "alphaSequence" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh_,
                dimensionedScalar
                (
                    "alphaSequence" + name(kth), 
                    dimensionSet(0,0,0,0,0,0,0), 
                    SMALL
                )
            )
        );
    
        bubblePressureGrad_.set
        (
            kth,
            new volVectorField 
            (
                IOobject
                (
                    "bubblePressureGrad" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::NO_WRITE
                ),
                mesh_,
                dimensionedVector
                (
                    "bubblePressureGrad" + name(kth),
                    dimensionSet(1,-2,-2,0,0,0,0),
                    vector::zero
                )
            )
        );

        gradPhaseFraction_.set
        (
            kth,
            new tmp<volVectorField>
            (
                new volVectorField
                (
                    IOobject
                    (
                        "gradPhaseFraction" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE,
                        false
                    ),
                    mesh_,
                    dimensionedVector
                    (
                        "gradPhaseFraction" + name(kth), 
                        dimensionSet(0,-1,0,0,0,0,0), 
                        Zero
                    )
                )
            )
        );

        particleU_.set
        (
            kth,
            new volVectorField
            (
                IOobject
                (
                    "particleU" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh_,
                dimensionedVector
                (
                    "particleU" + name(kth),
                    dimensionSet(0,1,-1,0,0,0,0),
                    vector::zero
                )
            )
        );
    
        Info<< "Initializing field " << "SU" << kth << nl; 
        SU_.set
        (
            kth,
            new volVectorField
            (
                IOobject
                (
                    "SU" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::NO_WRITE
                ),
                mesh_,
                vector::zero
            )
        );
    }
        
    //if (breakage_ == true)
    //{
    //    breakagePtr_ = breakageModel::New
    //    (
    //        MOMProperties.subDict
    //        (
    //            "breakageModel"
    //        ),
    //        mesh_
    //    );

    //    DSDPtr_ = daughterSizeDistributionModel::New
    //    (
    //        MOMProperties.subDict
    //        (
    //            "DSDModel"
    //        ),
    //        mesh_
    //    );
    //}

    //if (coalescence_ == true)
    //{
    //    coalescencePtr_ = coalescenceModel::New
    //    (
    //        MOMProperties.subDict
    //        (
    //            "coalescenceModel"
    //        ),
    //        mesh_
    //    );
    //}

    //- Initialize boundary weights and abscissas
    momInvPtr_->initilizeWeiAbs
    (
        weis_,
        abs_,
        M_,
        numberOfAbs_
    );
}


Foam::autoPtr<Foam::MOM> Foam::MOM::New
(
    const fvMesh& mesh,
    const dictionary& MOMProperties
)
{
    const word modelType(MOMProperties.lookup("MOMModel"));

    Info<< "Selecting MOM model is " << modelType << endl;

    dictionaryConstructorTable::iterator cstrIter =
        dictionaryConstructorTablePtr_->find(modelType);

    if (cstrIter == dictionaryConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown MOMModel type "
            << modelType << nl << nl
            << "Valid MOM are : " << endl
            << dictionaryConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return autoPtr<MOM>
        (cstrIter()(mesh, MOMProperties));
}
