/******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:          D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:               The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#ifndef MOM_H
#define MOM_H

#include "dictionary.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "fvc.H"
#include "force.H"
#include "uniformDimensionedFields.H"
#include "momentsInversion.H"
#include "timeScheme.H"
#include "Vikas.H"


namespace Foam
{

class MOM
{

protected:

    //- Protected data

        //- Mesh
        const fvMesh& mesh_;

        //- Force model
        force forceFrame_;

        //- MOM dictionary
        dictionary MOMDictionary_;

        //- MOM model
        word momModel_;

        //- Switch for the acceleration terms
        Switch forceTerm_;

        //- Number of moments
        label pureMomentsNumber_;

        //- Number of abscissas
        scalar numberOfAbs_;

        //- Adaptive number of abscissas
        scalar nodes_;

        //- Mean Sauter diameter field
        tmp<volScalarField> d32_;

        //- Moments
        PtrList<volScalarField> M_;

        //- Phase volume fraction
        volScalarField alpha_;
        volScalarField beta_;

        //- Velocity polynomial coefficients, X
        volVectorField VPCU_;

        //- Velocity polynomial coefficients, Y
        volVectorField VPCV_;

        //- Velocity polynomial coefficients, Z
        volVectorField VPCW_;

        //- Density of the dispersed phase
        dimensionedScalar rhod_;

        //- Density of the continuous phase
        volScalarField rhoc_;

        //- Moments Xflux components for Vikas schemes
        PtrList<surfaceScalarField> mFluxX_;

        //- Moments Yflux components for Vikas schemes
        PtrList<surfaceScalarField> mFluxY_;

        //- Moments Zflux components for Vikas schemes
        PtrList<surfaceScalarField> mFluxZ_;

        //- Velocity of the dispersed phase
        volVectorField Ud_;

    //- Abscissas and weights, QMOM and EQMOM

        //- Weights cell value
        PtrList<volScalarField> weis_;

        //- Abscissas cell value
        PtrList<volScalarField> abs_;

        //- Dispersed phase fraction sequence
        PtrList<volScalarField> alphaSequence_;

        //- Dispersed phase pressure gradient
        PtrList<volVectorField> bubblePressureGrad_;

        //- Dispersed phase fraction gradient
        PtrList<tmp<volVectorField>> gradPhaseFraction_;

        //- Particles velocity
        PtrList<volVectorField> particleU_;

        //- Moments inversion algorithm
        autoPtr<momentsInversion> momInvPtr_;

        //- Time scheme pointer 
        autoPtr<timeScheme> timeSchemePtr_;

        //- Moments flux model
        Vikas realizableMom_;

        //- SU field
        PtrList<volVectorField> SU_;

        //- Bubble pressure coefficient
        dimensionedScalar BP_;

    //- Protected member functions

        //- Disallow copy construct
        MOM(const MOM&);

        //- Disallow default bitwise assignment
        void operator=(const MOM&);


public:

    //- Runtime type information
    TypeName("MOM");

        declareRunTimeSelectionTable
        (
            autoPtr,
            MOM,
            dictionary,
            (
                const fvMesh& mesh,
                const dictionary& MOMProperties
            ),
            (mesh, MOMProperties)
        );

        static autoPtr<MOM> New
        (
            const fvMesh& mesh,
            const dictionary& MOMProperties
        );


    //- Constructers

        MOM
        (
            const fvMesh& mesh,
            const dictionary& MOMProperties
        );

        virtual ~MOM()
        {}


    //- Member functions

        //- Solve the mixed moments transport equations
        void solve();

        //- Solve the homogenerous PBE
        void solve0D();

        //- TVD time stepping scheme
        void RK2SSP();

        //- Update VPC in cells 
        void updateVPC(const label& cellI);

        //- Update VPC in boundary
        void correctVPCboundary();

    //- Inline functions

        //- Return drag Kd()
        inline const PtrList<volScalarField>& Kd() const;
        
        //- Return lift force
        inline const PtrList<volVectorField>& liftF() const;
        
        //- Return wall lubrication force
        inline const PtrList<volVectorField>& wallLubF() const;
        
        //- Return turbulent dispersion force
        inline const PtrList<volVectorField>& turbulentDisF() const;
        
        //- Return moments
        inline const PtrList<volScalarField>& mom() const;
        
        //- Return continuous phase fraction
        inline const volScalarField& alphac() const;
        
        //- Return rhod
        inline const dimensionedScalar& rhod() const;
        
        //- Return rhoc
        inline const volScalarField& rhoc() const;

        //- Return weights
        inline const PtrList<volScalarField>& weights() const;
        
        //- Return abscissas
        inline const PtrList<volScalarField>& abscissas() const;
        
        //- Return the dispersed phase fraction
        inline const volScalarField& weightedAlpha() const;
        
        //- Return the dispersed phase fraction sequence
        inline const PtrList<volScalarField>& alphaSequence() const;

        //- Return dispersed phase velocities
        inline const volVectorField& particleU
        (
            const scalar& kth
        ) const;

    //- Virtual functions

        //- Update the abscissas and weights in cells 
        virtual void calculateAbs(const label& cellI) = 0;

        //- Update the conditioned velocities in cells for abscissas
        void updateVelocity(const label& cellI);
        
        //- Moments acceleration update
        void updateMomentsForce();
        
        //- Correct the boundary conditions
        void correctBoundaryAndCells();
        
        //- Sample mixed moments calculation
        void mixedMomentsUpdate(const label& cellI);
        
        //- Calculate breakage source term
        //virtual void calculateSource() = 0;
        
        void updatebubblePressureForce();
                
        //- Velocity ODE in cells
        void velocityODE
        (
            const label& cellI,
            const vector& Uc,
            const scalar& muc,
            const scalar& rhoc,
            const scalar& dT,
            const uniformDimensionedVectorField& g,
            const vector& curlUc,
            const vector* gradPhaseFrac,
            const vector* bubblePressureGrad,
            const scalar& k,
            const scalar& nut
        );


};


} // End namespace Foam


#include "MOMI.H"

#endif
