/*******************************************************************************
@GPBEFoam:      3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:       Dongyue Li (HZDR & Dyfluid, Programmer)
                Daniele Marchisio (Politecnico di Torino, Co-PI)
                Christian Hasse (TU Darmstadt, Co-PI)
                Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                oneWayGPBEFoam: open-source Eulerian-QBMM solvers for multiphase 
                flows. Computer Physics Communication, Submitted

@History:       ver 1.0, 07/31/18 

@Info:          The released Eulerian-QBMM library is based on OpenFOAM-5.x. 
                oneWayGPBEFoam can be employed to simulate one-way coupled 
                problems. twoWayGPBEFoam can be employed to simulate two-way 
                coupled problems.
*******************************************************************************/

#include "Vikas.H"
#include "fvc.H"
#include "fixedValueFvPatchFields.H"    
#include "zeroGradientFvPatchFields.H"  
#include "wallFvPatch.H"


void Foam::Vikas::correctFlux
(
    const PtrList<volScalarField>& weights,
    const PtrList<volScalarField>& abscissas,
    const PtrList<volScalarField>& M,
    PtrList<surfaceScalarField>& mFluxX,
    PtrList<surfaceScalarField>& mFluxY,
    PtrList<surfaceScalarField>& mFluxZ,
    const PtrList<volVectorField>& particleU,
    const label pureMomentsNumber,
    const label nodes
)
{
    for(int n = 0; n < weights.size(); n++)
    {
        abscissasfPos_[n].ref() = 
            fvc::interpolate(abscissas[n], Pos, schemeAbs);

        abscissasfNeg_[n].ref() = 
            fvc::interpolate(abscissas[n], Neg, schemeAbs);

        weightsfPos_[n].ref() = 
            fvc::interpolate(weights[n], Pos, schemeWei);

        weightsfNeg_[n].ref() = 
            fvc::interpolate(weights[n], Neg, schemeWei); 

        phiUPos_[n].ref() = 
            fvc::interpolate(particleU[n].component(0), Pos, schemePhi)
           *particleU[n].mesh().Sf().component(0);

        phiUNeg_[n].ref() = 
            fvc::interpolate(particleU[n].component(0), Neg, schemePhi)
           *particleU[n].mesh().Sf().component(0);

        phiVPos_[n].ref() = 
            fvc::interpolate(particleU[n].component(1), Pos, schemePhi)
           *particleU[n].mesh().Sf().component(1);

        phiVNeg_[n].ref() = 
            fvc::interpolate(particleU[n].component(1), Neg, schemePhi)
           *particleU[n].mesh().Sf().component(1);

        phiWPos_[n].ref() = 
            fvc::interpolate(particleU[n].component(2), Pos, schemePhi)
           *particleU[n].mesh().Sf().component(2);

        phiWNeg_[n].ref() = 
            fvc::interpolate(particleU[n].component(2), Neg, schemePhi)
           *particleU[n].mesh().Sf().component(2);

        UP_[n] = fvc::interpolate(particleU[n], Pos, schemeU);
        UN_[n] = fvc::interpolate(particleU[n], Pos, schemeU);
    }
    
    //- Internal faces
    forAll(M, kth)
    {
        iniZero(kth); 

        MfXPos_[kth].ref().dimensions().reset(mFluxX[kth].dimensions());
        MfXNeg_[kth].ref().dimensions().reset(mFluxX[kth].dimensions());
        MfYPos_[kth].ref().dimensions().reset(mFluxX[kth].dimensions());
        MfYNeg_[kth].ref().dimensions().reset(mFluxX[kth].dimensions());
        MfZPos_[kth].ref().dimensions().reset(mFluxX[kth].dimensions());
        MfZNeg_[kth].ref().dimensions().reset(mFluxX[kth].dimensions());

        //- Not necessary to calculate the boundary values
        //- for MfPos and MfNeg
        for(int n = 0; n < abscissas.size(); n++)
        {
            if (kth < pureMomentsNumber)
            {
                MfXPos_[kth].ref() == 
                    MfXPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth)
                   *max(phiUPos_[n](), phi_zero);

                MfXNeg_[kth].ref() == 
                    MfXNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth)
                   *min(phiUNeg_[n](), phi_zero);

                MfYPos_[kth].ref() == 
                    MfYPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth)
                   *max(phiVPos_[n](), phi_zero);

                MfYNeg_[kth].ref() == 
                    MfYNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth)
                   *min(phiVNeg_[n](), phi_zero);

                MfZPos_[kth].ref() == 
                    MfZPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth)
                   *max(phiWPos_[n](), phi_zero);

                MfZNeg_[kth].ref() == 
                    MfZNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth)
                   *min(phiWNeg_[n](), phi_zero);
            }

            if 
            (
                kth > pureMomentsNumber - 1
             && kth < pureMomentsNumber + nodes 
            )
            {
                MfXPos_[kth].ref() == 
                    MfXPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber)
                   *max(phiUPos_[n](), phi_zero)
                   *UP_[n].component(0);

                MfXNeg_[kth].ref() == 
                    MfXNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber)
                   *min(phiUNeg_[n](), phi_zero)
                   *UN_[n].component(0);

                MfYPos_[kth].ref() == 
                    MfYPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber)
                   *max(phiVPos_[n](), phi_zero)
                   *UP_[n].component(0);

                MfYNeg_[kth].ref() == 
                    MfYNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber)
                   *min(phiVNeg_[n](), phi_zero)
                   *UN_[n].component(0);

                MfZPos_[kth].ref() == 
                    MfZPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber)
                   *max(phiWPos_[n](), phi_zero)
                   *UP_[n].component(0);

                MfZNeg_[kth].ref() == 
                    MfZNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber)
                   *min(phiWNeg_[n](), phi_zero)
                   *UN_[n].component(0);
            }

            if 
            (
                kth > pureMomentsNumber - 1 + nodes 
             && kth < pureMomentsNumber + 2*nodes
            )
            // 9 10 11
            // if (kth > 8 && kth < 12)
            {
                MfXPos_[kth].ref() == 
                    MfXPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber - nodes)
                   *max(phiUPos_[n](), phi_zero)
                   *UP_[n].component(1);

                MfXNeg_[kth].ref() == 
                    MfXNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber - nodes)
                   *min(phiUNeg_[n](), phi_zero)
                   *UN_[n].component(1);

                MfYPos_[kth].ref() == 
                    MfYPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber - nodes)
                   *max(phiVPos_[n](), phi_zero)
                   *UP_[n].component(1);

                MfYNeg_[kth].ref() == 
                    MfYNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber - nodes)
                   *min(phiVNeg_[n](), phi_zero)
                   *UN_[n].component(1);

                MfZPos_[kth].ref() == 
                    MfZPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber - nodes)
                   *max(phiWPos_[n](), phi_zero)
                   *UP_[n].component(1);

                MfZNeg_[kth].ref() == 
                    MfZNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber - nodes)
                   *min(phiWNeg_[n](), phi_zero)
                   *UN_[n].component(1);
            }


            if 
            (
                kth > pureMomentsNumber - 1 + 2*nodes
             && kth < pureMomentsNumber + 3*nodes
            )
            // 12 13 14
            //if(kth > 11 && kth < 15)
            {
                MfXPos_[kth].ref() == 
                    MfXPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber - 2*nodes)
                   *max(phiUPos_[n](), phi_zero)
                   *UP_[n].component(2);

                MfXNeg_[kth].ref() == 
                    MfXNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber - 2*nodes)
                   *min(phiUNeg_[n](), phi_zero)
                   *UN_[n].component(2);

                MfYPos_[kth].ref() == 
                    MfYPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber - 2*nodes)
                   *max(phiVPos_[n](), phi_zero)
                   *UP_[n].component(2);

                MfYNeg_[kth].ref() == 
                    MfYNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber - 2*nodes)
                   *min(phiVNeg_[n](), phi_zero)
                   *UN_[n].component(2);

                MfZPos_[kth].ref() == 
                    MfZPos_[kth]() + weightsfPos_[n]()
                   *pow(abscissasfPos_[n](), kth - pureMomentsNumber - 2*nodes)
                   *max(phiWPos_[n](), phi_zero)
                   *UP_[n].component(2);

                MfZNeg_[kth].ref() == 
                    MfZNeg_[kth]() + weightsfNeg_[n]()
                   *pow(abscissasfNeg_[n](), kth - pureMomentsNumber - 2*nodes)
                   *min(phiWNeg_[n](), phi_zero)
                   *UN_[n].component(2);
            }
        }

        mFluxX[kth] = MfXNeg_[kth]() + MfXPos_[kth]();
        mFluxY[kth] = MfYNeg_[kth]() + MfYPos_[kth]();
        mFluxZ[kth] = MfZNeg_[kth]() + MfZPos_[kth]();
    }

    const fvPatchList& patches = mesh_.boundary();

    //- Boundary patches
    forAll(mFluxX[0].boundaryField(), i)
    {
        if 
        (
            isA<fixedValueFvPatchScalarField>
            (
                M[0].boundaryField()[i]
            )
        )
        {
            forAll(mFluxX[0].boundaryField()[i], face)
            {
                forAll(mFluxX, kth)
                {
                    if (kth < pureMomentsNumber)
                    {
                        mFluxX[kth].boundaryFieldRef()[i][face] = 0.0;
                        mFluxY[kth].boundaryFieldRef()[i][face] = 0.0;
                        mFluxZ[kth].boundaryFieldRef()[i][face] = 0.0;

                        for (int n = 0; n < abscissas.size(); n++)
                        {
                            mFluxX[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth
                               )
                               *particleU[n].boundaryField()[i][face].x()
                               *mesh_.Sf().boundaryField()[i][face].x();

                            mFluxY[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth
                               )
                               *particleU[n].boundaryField()[i][face].y()
                               *mesh_.Sf().boundaryField()[i][face].y();

                            mFluxZ[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth
                               )
                               *particleU[n].boundaryField()[i][face].z()
                               *mesh_.Sf().boundaryField()[i][face].z();
                        }
                    }

                    if 
                    (
                        kth > pureMomentsNumber - 1
                     && kth < pureMomentsNumber + nodes
                    )
                    // 6 7 8
                    //if (kth == 6 || kth == 7 || kth == 8)
                    {
                        mFluxX[kth].boundaryFieldRef()[i][face] = 0.0;
                        mFluxY[kth].boundaryFieldRef()[i][face] = 0.0;
                        mFluxZ[kth].boundaryFieldRef()[i][face] = 0.0;

                        for (int n = 0; n < abscissas.size(); n++)
                        {
                            mFluxX[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber
                               )
                               *particleU[n].boundaryField()[i][face].x()
                               *particleU[n].boundaryField()[i][face].x()
                               *mesh_.Sf().boundaryField()[i][face].x();

                            mFluxY[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber

                               )
                               *particleU[n].boundaryField()[i][face].x()
                               *particleU[n].boundaryField()[i][face].y()
                               *mesh_.Sf().boundaryField()[i][face].y();

                            mFluxZ[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber

                               )
                               *particleU[n].boundaryField()[i][face].x()
                               *particleU[n].boundaryField()[i][face].z()
                               *mesh_.Sf().boundaryField()[i][face].z();
                        }
                    }

                    if 
                    (
                        kth > pureMomentsNumber - 1 + nodes
                     && kth < pureMomentsNumber + 2*nodes
                    )
                    {
                        mFluxX[kth].boundaryFieldRef()[i][face] = 0.0;
                        mFluxY[kth].boundaryFieldRef()[i][face] = 0.0;
                        mFluxZ[kth].boundaryFieldRef()[i][face] = 0.0;

                        for (int n = 0; n < abscissas.size(); n++)
                        {
                            mFluxX[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber - nodes
                               )
                               *particleU[n].boundaryField()[i][face].y()
                               *particleU[n].boundaryField()[i][face].x()
                               *mesh_.Sf().boundaryField()[i][face].x();
                    
                            mFluxY[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber - nodes

                               )
                               *particleU[n].boundaryField()[i][face].y()
                               *particleU[n].boundaryField()[i][face].y()
                               *mesh_.Sf().boundaryField()[i][face].y();

                            mFluxZ[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber - nodes

                               )
                               *particleU[n].boundaryField()[i][face].y()
                               *particleU[n].boundaryField()[i][face].z()
                               *mesh_.Sf().boundaryField()[i][face].z();
                        }
                    }

                    if 
                    (
                        kth > pureMomentsNumber - 1 + 2*nodes
                     && kth < pureMomentsNumber + 3*nodes
                    )
                    {
                        mFluxX[kth].boundaryFieldRef()[i][face] = 0.0;
                        mFluxY[kth].boundaryFieldRef()[i][face] = 0.0;
                        mFluxZ[kth].boundaryFieldRef()[i][face] = 0.0;

                        for (int n = 0; n < abscissas.size(); n++)
                        {
                            mFluxX[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber - 2*nodes
                               )
                               *particleU[n].boundaryField()[i][face].z()
                               *particleU[n].boundaryField()[i][face].x()
                               *mesh_.Sf().boundaryField()[i][face].x();
                    
                            mFluxY[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber - 2*nodes

                               )
                               *particleU[n].boundaryField()[i][face].z()
                               *particleU[n].boundaryField()[i][face].y()
                               *mesh_.Sf().boundaryField()[i][face].y();

                            mFluxZ[kth].boundaryFieldRef()[i][face] +=      
                                weights[n].boundaryField()[i][face]
                               *pow
                               (
                                    abscissas[n].boundaryField()[i][face], 
                                    kth - pureMomentsNumber - 2*nodes

                               )
                               *particleU[n].boundaryField()[i][face].z()
                               *particleU[n].boundaryField()[i][face].z()
                               *mesh_.Sf().boundaryField()[i][face].z();
                        }
                    }
                }
            }
        }

        //- wedge boundary
        if
        (
            patches[i].name() == "walls"
          || 
            patches[i].name() == "front"
          || 
            patches[i].name() == "back"
        )
        {
            forAll(mFluxX, kth)
            {
                forAll(mFluxX[0].boundaryField()[i], face)
                {
                    mFluxX[kth].boundaryFieldRef()[i][face] = 0.0;
                    mFluxY[kth].boundaryFieldRef()[i][face] = 0.0;
                    mFluxZ[kth].boundaryFieldRef()[i][face] = 0.0;
                }
            }
        }
    }
}


void Foam::Vikas::iniZero(const label kth)
{
    MfXPos_[kth].ref().primitiveFieldRef() = 0.0;
    MfXNeg_[kth].ref().primitiveFieldRef() = 0.0;
    MfYPos_[kth].ref().primitiveFieldRef() = 0.0;
    MfYNeg_[kth].ref().primitiveFieldRef() = 0.0;
    MfZPos_[kth].ref().primitiveFieldRef() = 0.0;
    MfZNeg_[kth].ref().primitiveFieldRef() = 0.0;
    MfXPos_[kth].ref().boundaryFieldRef() = 0.0;
    MfXNeg_[kth].ref().boundaryFieldRef() = 0.0;
    MfYPos_[kth].ref().boundaryFieldRef() = 0.0;
    MfYNeg_[kth].ref().boundaryFieldRef() = 0.0;
    MfZPos_[kth].ref().boundaryFieldRef() = 0.0;
    MfZNeg_[kth].ref().boundaryFieldRef() = 0.0;
}


Foam::Vikas::Vikas
(
    const fvMesh& mesh,
    const scalar numberOfMom,
    const scalar numberOfAbs
)
:
    mesh_(mesh),
    schemeAbs("interpolate(abscissas)"),
    schemeWei("interpolate(weights)"),
    schemeU("interpolate(QBMMU)"),
    schemePhi("interpolate(phi)"),
    Pos
    (
        IOobject
        (
            "Pos",
            mesh_.time().timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        mesh_,
        dimensionedScalar("Pos", dimless, 0.1)
    ),
    Neg
    (
        IOobject
        (
            "Neg",
            mesh_.time().timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        mesh_,
        dimensionedScalar("Neg", dimless, -0.1)
    ),
    phiUPos_(numberOfAbs),
    phiUNeg_(numberOfAbs),
    phiVPos_(numberOfAbs),
    phiVNeg_(numberOfAbs),
    phiWPos_(numberOfAbs),
    phiWNeg_(numberOfAbs),
    phi_zero
    (
        "phi_zero", 
        dimVelocity*dimArea,
        0.0
    ),
    MfXPos_(numberOfMom),
    MfXNeg_(numberOfMom),
    MfYPos_(numberOfMom),
    MfYNeg_(numberOfMom),
    MfZPos_(numberOfMom),
    MfZNeg_(numberOfMom),
    abscissasfPos_(numberOfAbs),
    abscissasfNeg_(numberOfAbs),
    weightsfPos_(numberOfAbs),
    weightsfNeg_(numberOfAbs),
    UP_(numberOfAbs),
    UN_(numberOfAbs)
{
    Info<<"\nVikas constructing function\n\n";
    forAll(UP_, kth)
    {
        abscissasfNeg_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "abscissasfNeg" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "abscissasfNeg" + name(kth),
                        dimensionSet(0,1,0,0,0,0,0),
                        scalar(0.0)
                    )
                    //calculatedFvPatchScalarField::typeName
                )
            )
        );

        abscissasfPos_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "abscissasfPos" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "abscissasfNeg" + name(kth),
                        dimensionSet(0,1,0,0,0,0,0),
                        scalar(0.0)
                    )
                    //M_[kth].boundaryField().types()//for cyclic
                )
            )
        );

        weightsfNeg_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "weightsfNeg" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "weightsfNeg" + name(kth),
                        dimensionSet(0,-3,0,0,0,0,0),
                        scalar(0.0)
                    )
                )
            )
        );

        weightsfPos_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "weightsfPos" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "weightsfNeg" + name(kth),
                        dimensionSet(0,-3,0,0,0,0,0),
                        scalar(0.0)
                    )
                )
            )
        );

        phiUPos_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "phiUPos" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "phiUPos" + name(kth),
                        dimensionSet(0,3,-1,0,0,0,0),
                        0.0
                    )
                )
            )
        );

        phiUNeg_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "phiUNeg" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "phiUNeg" + name(kth),
                        dimensionSet(0,3,-1,0,0,0,0),
                        0.0
                    )
                )
            )
        );
        
        phiVPos_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "phiVPos" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "phiVPos" + name(kth),
                        dimensionSet(0,3,-1,0,0,0,0),
                        0.0
                    )
                )
            )
        );

        phiVNeg_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "phiVNeg" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "phiVNeg" + name(kth),
                        dimensionSet(0,3,-1,0,0,0,0),
                        0.0
                    )
                )
            )
        );

        phiWNeg_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "phiWNeg" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "phiWNeg" + name(kth),
                        dimensionSet(0,3,-1,0,0,0,0),
                        0.0
                    )
                )
            )
        );

        phiWPos_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField
                (
                    IOobject
                    (
                        "phiWPos" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "phiWPos" + name(kth),
                        dimensionSet(0,3,-1,0,0,0,0),
                        0.0
                    )
                )
            )
        );

        UP_.set
        (
            kth,
            new surfaceVectorField
            (
                IOobject
                (
                    "UP" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::NO_WRITE
                ),
                mesh_,
                dimensionedVector
                (
                    "UP" + name(kth),
                    dimensionSet(0,1,-1,0,0,0,0),
                    vector::zero
                )
            )
        );

        UN_.set
        (
            kth,
            new surfaceVectorField
            (
                IOobject
                (
                    "UN" + name(kth),
                    mesh_.time().timeName(),
                    mesh_,
                    IOobject::NO_READ,
                    IOobject::NO_WRITE
                ),
                mesh_,
                dimensionedVector
                (
                    "UN" + name(kth),
                    dimensionSet(0,1,-1,0,0,0,0),
                    vector::zero
                )
            )
        );
    }

    forAll(MfXNeg_, kth)
    {
        MfXNeg_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField 
                (
                    IOobject
                    (
                        "MfXNeg" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE,
                        false
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "MfXNeg" + name(kth),
                        dimless,
                        0.0
                    )
                    //M_[kth].boundaryField().types()//for cyclic
                )
            )
        );

        MfXPos_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField 
                (
                    IOobject
                    (
                        "MfXPos" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE,
                                                    false
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "MfXPos" + name(kth),
                        dimless,
                        0.0
                    )
                    //M_[kth].boundaryField().types()//for cyclic
                )
            )
        );

        //- Dimensions updated in correctFlux()
        MfYNeg_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField 
                (
                    IOobject
                    (
                        "MfYNeg" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE,
                        false
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "MfYNeg" + name(kth),
                        dimless,
                        0.0
                    )
                    //M_[kth].boundaryField().types()//for cyclic
                )
            )
        );

        MfYPos_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField 
                (
                    IOobject
                    (
                        "MfYPos" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE,
                        false
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "MfYPos" + name(kth),
                        dimless,
                        0.0
                    )
                    //M_[kth].boundaryField().types()//for cyclic
                )
            )
        );

        //- Dimensions updated in correctFlux()
        MfZNeg_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField 
                (
                    IOobject
                    (
                        "MfZNeg" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE, 
                        false
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "MfZNeg" + name(kth),
                        dimless,
                        0.0
                    )
                    //M_[kth].boundaryField().types()//for cyclic
                )
            )
        );

        MfZPos_.set
        (
            kth,
            new tmp<surfaceScalarField>
            (
                new surfaceScalarField 
                (
                    IOobject
                    (
                        "MfZPos" + name(kth),
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::NO_WRITE,
                        false
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "MfZPos" + name(kth),
                        dimless,
                        0.0
                    )
                    //M_[kth].boundaryField().types()//for cyclic
                )
            )
        );
    }    
}
