/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "momentsInversion.H"
#include "fixedValueFvPatchFields.H"	
#include "zeroGradientFvPatchFields.H"	


namespace Foam
{
    defineTypeNameAndDebug(momentsInversion, 0);
    defineRunTimeSelectionTable(momentsInversion, dictionary);
}


void Foam::momentsInversion::solve
(
    PtrList<volScalarField>& weights,
    PtrList<volScalarField>& abscissas,
    PtrList<volScalarField>& moments,
    const label& numberOfAbs,
    const label& cellI
)
{
    scalar mm[2*numberOfAbs];
    
    for(int i = 0; i < 2*numberOfAbs; i++)
    {
    	mm[i] = moments[i][cellI]/moments[0][cellI];
    }

    //- Calculate matrix in pdCore algorithm
    scalarSquareMatrix jacobi(core(numberOfAbs, mm));

    eigenSolver jacobiEig(jacobi, true);

    for (label i = 0; i < numberOfAbs; i++)
    {
        weights[i][cellI] = 
            moments[0][cellI]*sqr(jacobiEig.eigenvectors()[0][i]);

    	abscissas[i][cellI] = jacobiEig.eigenvaluesRe()[i];
    
    	if (abscissas[i][cellI] < 0.0)
    	{
            FatalErrorIn("size < 0") << exit(FatalError);
        }
    }
}


void Foam::momentsInversion::solveBoundary
(
    PtrList<volScalarField>& weights,
    PtrList<volScalarField>& abscissas,
    PtrList<volScalarField>& moments,
    const label& numberOfAbs
)
{
    //- Update zero gradient weights
    const fvPatchList& patches = mesh_.boundary();

    forAll(moments[0].boundaryField(), patch)
    {
        if 
        (
            isA<zeroGradientFvPatchScalarField>
            (
                moments[0].boundaryField()[patch]
            )
        )
        {
            const fvPatch& currPatch = patches[patch];

            forAll(currPatch, f)
            {
                forAll(weights, kth)
                {
                    label faceCelli = currPatch.faceCells()[f];

                    weights[kth].boundaryFieldRef()[patch][f] =
                        weights[kth][faceCelli];

                    abscissas[kth].boundaryFieldRef()[patch][f] =
                        abscissas[kth][faceCelli];
                }
            }
        }

        //- Processors boundary
        else if
        (
            ! isA<fixedValueFvPatchScalarField>
            (
                moments[0].boundaryField()[patch]
            )
        )
        {
            forAll(moments[0].boundaryField()[patch], face)
            {
                scalar mm[2*numberOfAbs];
    
                for(int i = 0; i < 2*numberOfAbs; i++)
                {
                    mm[i] = 
                        moments[i].boundaryField()[patch][face]
                       /moments[0].boundaryField()[patch][face];
                }

                scalarSquareMatrix jacobi(core(numberOfAbs, mm));

                eigenSolver jacobiEig(jacobi, true);

                for (label i = 0; i < numberOfAbs; i++)
                {
                    scalar& wei = 
                        weights[i].boundaryFieldRef()[patch][face];

                    scalar& abs = 
                        abscissas[i].boundaryFieldRef()[patch][face];

                    wei = moments[0].boundaryField()[patch][face]
                       *sqr(jacobiEig.eigenvectors()[0][i]);

                    abs = jacobiEig.eigenvaluesRe()[i];
                
                    if (abs < 0.0)
                    {
                        FatalErrorIn
                        (
                            "processor boundary size < 0"
                        )   
                        << exit(FatalError);
                    }
                }
            }
        }
    }
}


void Foam::momentsInversion::initilizeWeiAbs
(
    PtrList<volScalarField>& weights,
    PtrList<volScalarField>& abscissas,
    PtrList<volScalarField>& moments,
    const label& numberOfAbs
)
{
    const fvPatchList& patches = mesh_.boundary();

    forAll(moments[0].boundaryField(), patch)
    {
        if 
        (
            isA<fixedValueFvPatchScalarField>
            (
                moments[0].boundaryField()[patch]
            )
        )
        {
	    forAll(moments[0].boundaryField()[patch], face)
            {
                scalar mm[2*numberOfAbs];
    
                for(int i = 0; i < 2*numberOfAbs; i++)
                {
                    mm[i] = 
                        moments[i].boundaryFieldRef()[patch][face]
                       /moments[0].boundaryFieldRef()[patch][face]
;
                }

                scalarSquareMatrix jacobi(core(numberOfAbs, mm));

                eigenSolver jacobiEig(jacobi, true);

                for (label i = 0; i < numberOfAbs; i++)
                {
                    scalar& wei = 
                        weights[i].boundaryFieldRef()[patch][face];

                    scalar& abs = 
                        abscissas[i].boundaryFieldRef()[patch][face];

                    wei = moments[0].boundaryFieldRef()[patch][face]
                       *sqr(jacobiEig.eigenvectors()[0][i]);

                    abs = jacobiEig.eigenvaluesRe()[i];
                
                    if (abs < 0.0)
                    {
                        FatalErrorIn
                        (
                            "boundary size < 0"
                        )   
                        << exit(FatalError);
                    }
                }
            }
        }
        else 
        {
            const fvPatch& currPatch = patches[patch];

            forAll(currPatch, f)
            {
                forAll(weights, kth)
                {
                    label faceCelli = currPatch.faceCells()[f];

                    weights[kth].boundaryFieldRef()[patch][f] =
                        weights[kth][faceCelli];

                    abscissas[kth].boundaryFieldRef()[patch][f] =
                        abscissas[kth][faceCelli];
                }
            }
        }
    }
}


Foam::momentsInversion::momentsInversion
(
    const word& momInvModel,
    const fvMesh& mesh
)
:
    momInvModel_(momInvModel),
    mesh_(mesh)
{}


Foam::autoPtr<Foam::momentsInversion> Foam::momentsInversion::New
(
    const word& momInvModel,
    const fvMesh& mesh
)
{
    Info<< "Foam::momentsInversion::New FUNCTION" << endl;

    const word modelType(momInvModel);

    Info<< "Selecting momentsInversion model is " << modelType << endl;

    dictionaryConstructorTable::iterator cstrIter =
        dictionaryConstructorTablePtr_->find(modelType);


    if (cstrIter == dictionaryConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown momentsInversionModel type "
            << modelType << nl << nl
            << "Valid momentsInversion are : " << endl
            << dictionaryConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return autoPtr<momentsInversion>
        (cstrIter()(momInvModel, mesh));
}
