/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "pd.H"
#include "addToRunTimeSelectionTable.H"


namespace Foam
{
    defineTypeNameAndDebug(pd, 0);

    addToRunTimeSelectionTable
    (
        momentsInversion,
        pd,
        dictionary
    );
}


Foam::scalarSquareMatrix Foam::pd::core
(
    const label n,
    const scalar* mom
)
{
    scalar p[2*n + 1][2*n + 1];
 
    double* zeta = new double [2*n];

    weisBD_ = new double[n];
    absBD_ = new double[n];
	
    for (int i = 0; i < 2*n + 1; i++)
    {
        for (int j = 0; j < 2*n + 1; j++)
        {
	        p[i][j]=0.0;
        }
    }

    p[0][0] = 1.0;
    p[0][1] = 1.0;

    for (int i = 1; i < 2*n; i++)
    {
        p[i][1] = Foam::pow(-1.0, scalar(i))*mom[i];
    }

    for (int j = 2; j < 2*n + 1; j++)
    {
        for (int i = 0; i < 2*n + 2 - j; i++)
        {
            p[i][j] = p[0][j - 1]*p[i + 1][j - 2]
                - p[0][j - 2]*p[i + 1][j - 1];
    	}
    }

    for(int i = 0; i < 2*n; i++)
    {
        zeta[i] = 0.0;
    }

    for(int i = 1; i < 2*n; i++)
    {
        if (p[0][i]*p[0][i-1] > 0.0)
        {
    	    zeta[i] = p[0][i + 1]/(p[0][i]*p[0][i - 1]);
        }
        else
        {
            zeta[1] = 0.0;
        }
    }

    double a[n], b[n - 1], c[n - 1];

    for (int i = 0; i < n; i++)
    {
        a[i] = 0.0;
    }
	
    for (int i = 0; i < n - 1; i++)
    {
	b[i] = c[i] = 0.0;
    }

    for (int i = 1; i <= n; i++)
    {
        a[i - 1] = zeta[2*i - 1] + zeta[2*i - 2];
    }

    for (int i = 1; i <= n - 1; i++)
    {
	b[i - 1] = zeta[2*i]*zeta[2*i - 1];
    }

    for (int i = 1; i <= n - 1; i++)
    {
	//- Check fabs?
	c[i - 1] = Foam::sqrt(fabs(b[i - 1]));
    }
 
    scalarSquareMatrix jacobi(n, scalar(0));

    for (label i = 0; i < n; i++)
    {
        jacobi[i][i] = a[i];
    }
    
    for (label i = 0; i < n - 1; i++)
    {
        //- Check mag?
        jacobi[i][i+1] = -Foam::sqrt(Foam::mag(b[i]));
        jacobi[i+1][i] = -Foam::sqrt(Foam::mag(b[i]));
    }

    delete [] zeta;
    delete [] weisBD_;
    delete [] absBD_;

    return jacobi;
}


Foam::pd::pd
(
    const word& momInvModel,
    const fvMesh& mesh
)
:
    momentsInversion(momInvModel, mesh),
    weisBD_(nullptr),
    absBD_(nullptr)
{}
