/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "wheeler.H"
#include "addToRunTimeSelectionTable.H"


namespace Foam
{
    defineTypeNameAndDebug(wheeler, 0);

    addToRunTimeSelectionTable
    (
        momentsInversion,
        wheeler,
        dictionary
    );
}


Foam::scalarSquareMatrix Foam::wheeler::core
(
    const label n,
    const scalar* mom
)
{
    scalarDiagonalMatrix a(n, scalar(0));
    scalarDiagonalMatrix b(n, scalar(0));
    scalarSquareMatrix sig(2*n + 1, scalar(0));

    for(label i = 1; i < 2*n+1; i++)
    {
        sig[1][i] = mom[i-1];
    }

    a[0] = mom[1]/mom[0];
    b[0] = scalar(0);

    for(label k = 2; k < n + 1; k++)
    {
        for(label l = k; l < 2*n - k + 3; l++)
        {
            sig[k][l] = sig[k-1][l+1] - a[k-2]*sig[k-1][l] 
                        - b[k-2]*sig[k-2][l];
        }
        a[k-1] = sig[k][k+1]/sig[k][k] - sig[k-1][k]/sig[k-1][k-1];
        b[k-1] = sig[k][k]/sig[k-1][k-1];
    }

    scalarSquareMatrix z(n, scalar(0));

    for(label i = 0; i < n - 1; i++)
    {
        z[i][i] = a[i];
        z[i][i+1] = Foam::sqrt(b[i+1]);
        z[i+1][i] = z[i][i+1];
    }

    z[n-1][n-1] = a[n-1];

    return z;
}


Foam::wheeler::wheeler
(
    const word& momInvModel,
    const fvMesh& mesh
)
:
    momentsInversion(momInvModel, mesh),
    weisBD_(nullptr),
    absBD_(nullptr)
{}
