/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:          D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:               The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#ifndef parcel_H
#define parcel_H

#include "scalar.H"
#include "scalarMatrices.H"
#include "complex.H"

namespace Foam
{


class parcel
{
    //- Particle cell number
    label cellI_;

    //- Particle velocity
    vector Ud_;

    //- Continuous velocity
    vector Uc_;

    //- Curl continuous phase velocity
    vector curlUc_;

    //- Kinetic energy
    scalar k_;

    //- Particle density
    scalar rhod_;

    //- Continuous phase density
    scalar rhoc_;

    //- Continuous phase viscosity
    scalar muc_;

    //- Continuous phase turbulent viscosity
    scalar nut_;

    //- Particle diameter
    scalar diameter_;

    //- Phase fraction
    scalar phaseFraction_;

    //- Phase fraction
    scalar alpha_;

    //- Grad phase fraction
    const vector* gradPhaseFraction_;

    //- Label of kth
    label kth_;

    //- Slip velocity
    vector Ur_;

    //- Magnitude of the slip velocity
    scalar magUr_;


    public:

    //- Constructor

    parcel
    (
        const label& cellI,
        const vector& Ud,
        const vector& Uc,
        const vector& curlUc,
        const scalar& k,
        const scalar& rhod,
        const scalar& rhoc,
        const scalar& muc,
        const scalar& nut,
        const scalar& diameter,
        const scalar& phaseFraction,
        const scalar& alpha,
        const vector* gradPhaseFraction,
        const label& kth
    ):
        cellI_(cellI),
        Ud_(Ud),
        Uc_(Uc),
        curlUc_(curlUc),
        k_(k),
        rhod_(rhod),
        rhoc_(rhoc),
        muc_(muc),
        nut_(nut),
        diameter_(diameter),
        phaseFraction_(phaseFraction),
        alpha_(alpha),
        gradPhaseFraction_(nullptr),
        kth_(kth),
        Ur_(Uc - Ud),
        magUr_(mag(Uc - Ud))
    {
        gradPhaseFraction_ = gradPhaseFraction;
    }

    //- Inline functions    

    //- Return cell number
    const label& cellI() const;

    //- Return Particle velocity
    const vector& Ud() const;

    //- Return Continuous velocity
    const vector& Uc() const;

    //- Return k 
    const scalar& k() const;

    //- Return curl Continuous velocity
    const vector& curlUc() const;

    //- Return Slip velocity
    const vector& Ur() const;

    //- Return Magnitude of the slip velocity
    const scalar& magUr() const;

    //- Return Particle density
    const scalar& rhod() const;

    //- Return Continuous phase density
    const scalar& rhoc() const;

    //- Return Continuous phase viscosity
    const scalar& muc() const;

    //- Return Particle diameter
    const scalar& d() const;

    //- Return Phase fraction
    const scalar& phaseFraction() const;

    //- Return Phase fraction
    const scalar& alpha() const;

    //- Return turbulent viscosity
    const scalar& nut() const;

    //- Return Grad phase fraction
    const vector* gradPhaseFraction() const;

    //- Return Label of kth
    const label& kth() const;


}; // End nested class


};


#endif
