/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "Grace.H"
#include "addToRunTimeSelectionTable.H"


namespace Foam
{
    defineTypeNameAndDebug(Grace, 0);

    addToRunTimeSelectionTable
    (
        drag,
        Grace,
        dictionary
    );
}


void Foam::Grace::updateCdRe
(
    const label& cellI,
    const scalar& diameter,
    const scalar& Ur,
    const scalar& rhoc,
    const scalar& rhod,
    const scalar& muc,
    const scalar& alpha,
    const scalar& kth
) 
{
    scalar g = 9.81;
    scalar sigma = 0.072;
    scalar nuc = muc/rhoc;

    scalar Re(Ur*diameter*rhoc/muc);

    scalar Eo(mag(rhoc - rhod)*g*sqr(diameter)/sigma);

    scalar Mo(g*nuc*pow3(muc/sigma));

    scalar muref = 0.0009;

    scalar H
    (
        4.0/3.0*Eo*pow(Mo, -0.149)*pow(muc/muref, -0.14)
    );

    scalar J
    (
        neg(H - 59.3)*0.94*pow(H, 0.751)
      + pos(H - 59.3)*3.42*pow(H, 0.441)
    );

    scalar Ut
    (
        muc/rhoc/diameter*pow(Mo, -0.149)*(J - 0.857)
    );

    CdRe_[kth][cellI] = 4.0/3.0*(rhoc - rhod)/rhoc*g*diameter/sqr(Ut)*max(Re, residualRe_);

}


Foam::Grace::Grace
(
    const dictionary& dragModels,
    const fvMesh& mesh,
    const bool registerObject
)
:
    drag(dragModels, mesh, registerObject),
    residualRe_(readScalar(dragModels.lookup("residualRe")))
{}
