/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "IshiiZuber.H"
#include "addToRunTimeSelectionTable.H"


namespace Foam
{
    defineTypeNameAndDebug(IshiiZuber, 0);

    addToRunTimeSelectionTable
    (
        drag,
        IshiiZuber,
        dictionary
    );
}


void Foam::IshiiZuber::updateCdRe
(
    const label& cellI,
    const scalar& diameter,
    const scalar& Ur,
    const scalar& rhoc,
    const scalar& rhod,
    const scalar& muc,
    const scalar& alpha,
    const scalar& kth
) 
{
    scalar g = 9.81;
    scalar sigma = 0.072;
    scalar mud = 1e-3;
    scalar Re(Ur*diameter*rhoc/muc);
    scalar Eo(mag(rhoc - rhod)*g*sqr(diameter)/sigma);

    scalar muStar((mud + 0.4*muc)/(mud + muc));

    scalar muMix
    (
        muc
       *pow(max(1.0 - alpha, scalar(1e-3)), -2.5*muStar)
    );

    scalar ReM(Re*muc/muMix);

    scalar CdRe
    (
        pos0(1000 - ReM)*24.0*(scalar(1.0) + 0.15*pow(ReM, 0.687))
      + neg(1000 - ReM)*0.44*ReM
    );

    scalar F(max((muc/muMix)*sqrt(1.0 - alpha), 1e-3));

    scalar Ealpha((1.0 + 17.67*pow(F, 0.8571428))/(18.67*F));

    scalar CdReEllipse(Ealpha*0.6666*sqrt(Eo)*Re);

    CdRe_[kth][cellI] =
        pos0(CdReEllipse - CdRe)
       *min(CdReEllipse, Re*sqr(1.0 - alpha)*2.66667)
      + neg(CdReEllipse - CdRe)*CdRe;
}


Foam::IshiiZuber::IshiiZuber
(
    const dictionary& dragModels,
    const fvMesh& mesh,
    const bool registerObject
)
:
    drag(dragModels, mesh, registerObject),
    residualRe_(readScalar(dragModels.lookup("residualRe")))
{}
