/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/


#include "drag.H"
#include "zeroGradientFvPatchFields.H"


namespace Foam
{
    defineTypeNameAndDebug(drag, 0);
    defineRunTimeSelectionTable(drag, dictionary);
}


Foam::drag::drag
(
    const dictionary& forceModels,
    const fvMesh& mesh,
    const bool registerObject
)
:
    regIOobject
    (
        IOobject
        (
            "drag",
            mesh.time().timeName(),
            mesh,
            IOobject::NO_READ,
            IOobject::NO_WRITE,
            registerObject
        )
    ),
    tau_
    (
        IOobject
        (
            "tau",
            mesh.time().timeName(),
            mesh,
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        mesh,
        dimensionedScalar
        (
            "tau", 
            dimensionSet(0,0,1,0,0,0,0), 
            0.0	
        ),
        zeroGradientFvPatchField<scalar>::typeName
    ),
    dragModels_(forceModels.subDict("dragModels")),
    CdRe_(readScalar(tau_.mesh().lookupObject<dictionary>("GPBEProperties").lookup("nodesNumber")))
{
    forAll(CdRe_, kth) 
    {
        CdRe_.set
        (
            kth,
            new volScalarField 
            (
        	IOobject
        	(
        	    "CdRe" + Foam::name(kth),
        	    mesh.time().timeName(),
        	    mesh,
        	    IOobject::NO_READ,
        	    IOobject::NO_WRITE
        	),
        	mesh,
        	scalar(0.0)
            )
        );
    }
}


bool Foam::drag::writeData(Ostream& os) const
{
    return os.good();
}


Foam::autoPtr<Foam::drag> Foam::drag::New
(
    const dictionary& forceModels,
    const fvMesh& mesh
)
{
    const word modelType(forceModels.subDict("dragModels").lookup("dragModel"));

    Info << "Selecting drag model " << modelType 
        << endl;

    dictionaryConstructorTable::iterator cstrIter =
        dictionaryConstructorTablePtr_->find(modelType);

    if (cstrIter == dictionaryConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown drag type "
            << modelType << nl << nl
            << "Valid drags are : " << endl
            << dictionaryConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return autoPtr<drag>
        (cstrIter()(forceModels, mesh, true));
}
