/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#ifndef drag_H
#define drag_H

#include "dictionary.H"
#include "runTimeSelectionTables.H"
#include "volFields.H"


namespace Foam
{

class fvMesh;

class drag
:
    public regIOobject
{

protected:

    //- Protected data
		
        //- Characteristic time scale
        volScalarField tau_;

        //- Dictionary for drag model
        dictionary dragModels_;	

        //- CdRe PtrList
        PtrList<volScalarField> CdRe_;


    //- Protected member functions

        //- Disallow copy construct
        drag(const drag&);

        //- Disallow default bitwise assignment
        void operator=(const drag&);


public:

    //- Runtime type information
    TypeName("drag");

        //- Declare run-time constructor selection table
        declareRunTimeSelectionTable
        (
            autoPtr,
            drag,
            dictionary,
            (
                const dictionary& forceModels,
                const fvMesh& mesh,
                const bool registerObject
            ),
            (forceModels, mesh, registerObject)
        );

        //- Return a reference to the selected drag model
        static autoPtr<drag> New
        (
            const dictionary& forceModels,
            const fvMesh& mesh
        );


    //- Constructers

        drag
        (
            const dictionary& forceModels,
            const fvMesh& mesh,
            const bool registerObject
        );

        virtual ~drag()
        {}


    //- Member functions

        //- Calculate CdRe
        virtual void updateCdRe
        (
            const label& cellI,
            const scalar& diameter,
            const scalar& Ur,
            const scalar& rhoc,
            const scalar& rhod,
            const scalar& muc,
            const scalar& alpha,
            const scalar& kth
        ) = 0;

        //- Return CdRe
        inline const scalar& CdRe
        (
            const scalar& kth,
            const label& cellI
        ) const
        {
            return CdRe_[kth][cellI];
        }

        //- Dummy write for regIOobject
        bool writeData(Ostream& os) const;

};


} // End namespace Foam


#endif
