/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "force.H"
#include "zeroGradientFvPatchFields.H"


Foam::force::force
(
    const dictionary& MOMProperties,
    const fvMesh& mesh
)
:
    forceModels_(MOMProperties.subDict("forceModels")),
    dragModelPtr_
    (
        drag::New
        (
            forceModels_, 
            mesh
        )
    ),
    liftModelPtr_
    (
    	liftModels::lift::New
    	(
    	    forceModels_, 
    	    mesh
    	)
    ),
    wallLubModelPtr_
    (
    	wallLubricationModels::wallLubrication::New
    	(
    	    forceModels_, 
    	    mesh
    	)
    ),
    turbulentDisModelPtr_
    (
    	turbulentDispersionModels::turbulentDispersion::New
    	(
    	    forceModels_, 
    	    mesh
    	)
    ),
    Kd_(readScalar(MOMProperties.lookup("nodesNumber"))),
    liftF_(readScalar(MOMProperties.lookup("nodesNumber"))),
    wallLubF_(readScalar(MOMProperties.lookup("nodesNumber"))),
    turbulentDisF_(readScalar(MOMProperties.lookup("nodesNumber"))),
    Re_(readScalar(MOMProperties.lookup("nodesNumber")))
{
    forAll(Kd_, kth)
    {
        Kd_.set
        (
            kth,
            new volScalarField 
            (
                IOobject
                (
                    "Kd" + name(kth),
                    mesh.time().timeName(),
                    mesh,
                    IOobject::NO_READ,
                    IOobject::NO_WRITE
                ),
                mesh,
                dimensionedScalar
                (
                    "Kd" + name(kth),
                    dimensionSet(1,-3,-1,0,0,0,0), 
                    SMALL 
                )
                //zeroGradientFvPatchField<scalar>::typeName
            )
        );

        liftF_.set
        (
            kth,
            new volVectorField 
            (
                IOobject
                (
                    "liftF" + name(kth),
                    mesh.time().timeName(),
                    mesh,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh,
                dimensionedVector
                (
                    "liftF" + name(kth),
                    dimensionSet(0,1,-2,0,0,0,0), 
                    Zero 
                )
                //zeroGradientFvPatchField<scalar>::typeName
            )
        );

        wallLubF_.set
        (
            kth,
            new volVectorField 
            (
                IOobject
                (
                    "wallLubF" + name(kth),
                    mesh.time().timeName(),
                    mesh,
                    IOobject::NO_READ,
                    IOobject::AUTO_WRITE
                ),
                mesh,
                dimensionedVector
                (
                    "wallLubF" + name(kth),
                    dimensionSet(0,1,-2,0,0,0,0), 
                    Zero
                )
                //zeroGradientFvPatchField<scalar>::typeName
            )
        );

        turbulentDisF_.set
        (
            kth,
            new volVectorField 
            (
                IOobject
                (
                    "turbulentDis" + name(kth),
                    mesh.time().timeName(),
                    mesh,
                    IOobject::NO_READ,
                    IOobject::NO_WRITE
                ),
                mesh,
                dimensionedVector
                (
                    "turbulentDisF" + name(kth),
                    dimensionSet(0,1,-2,0,0,0,0), 
                    Zero
                ),
                zeroGradientFvPatchField<scalar>::typeName
            )
        );

        Re_.set
        (
            kth,
            new volScalarField 
            (
                IOobject
                (
                    "Re" + name(kth),
                    mesh.time().timeName(),
                    mesh,
                    IOobject::NO_READ,
                    IOobject::NO_WRITE
                ),
                mesh,
                dimensionedScalar
                (
                    "Re" + name(kth),
                    dimless,
                    0.0
                ),
                zeroGradientFvPatchField<scalar>::typeName
            )
        );
    }
}
