/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#ifndef force_H
#define force_H

#include "drag.H"
#include "lift.H"
#include "wallLubrication.H"
#include "turbulentDispersion.H"
#include "autoPtr.H"


namespace Foam
{

class fvMesh;

class force
{

protected:

    //- Protected data

        //- Dictionary for force models
        dictionary forceModels_;	

        //- Drag model pointer
        autoPtr<drag> dragModelPtr_;

        //- Lift model pointer
        autoPtr<liftModels::lift> liftModelPtr_;
        
        //- Wall lubrication pointer
        autoPtr<wallLubricationModels::wallLubrication> wallLubModelPtr_;
        
        //- Turbulent dispersion pointer
        autoPtr<turbulentDispersionModels::turbulentDispersion> 
        turbulentDisModelPtr_;
        
        //- Drag Kd
        PtrList<volScalarField> Kd_;
        
        //- Lift force
        PtrList<volVectorField> liftF_;

        //- Wall lubrication force
        PtrList<volVectorField> wallLubF_;
        
        //- Turbulent dispersion force
        PtrList<volVectorField> turbulentDisF_;
        
        //- Reynold number for particles
        PtrList<volScalarField> Re_;

		
    //- Protected member functions

        //- Disallow copy construct
        force(const force&);

        //- Disallow default bitwise assignment
        void operator=(const force&);


public:

    //- Constructers

        force
        (
            const dictionary& MOMProperties,
            const fvMesh& mesh
        );

        virtual ~force()
        {}


    //- Inline functions

            //- Return drag Kd
            inline const PtrList<volScalarField>& Kd() const;
            
            //- Return lift force
            inline const PtrList<volVectorField>& liftF() const;
            
            //- Return wall lubrication force 
            inline const PtrList<volVectorField>& wallLubF() const;
            
            //- Return turbulent dispersion force 
            inline const PtrList<volVectorField>& turbulentDisF() const;
            
            //- Return drag Kd
            template<class Type>
            inline void updateRe(const Type& p);


    //- Member functions
		
        //- Update force coefficients
        template<class Type>
        void updateCoefficients(const Type& p);
};


} // End namespace Foam


#include "forceI.H"
#ifdef NoRepository
    #include "forceTemplate.C"
#endif

#endif
