/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:          D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:               The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/


template<class Type>
void Foam::force::updateCoefficients(const Type& p)
{
    //- Update the Reynolds number
    updateRe(p);

    //- Update drag CdRe
    dragModelPtr_->updateCdRe
    (
        p.cellI(), 
        p.d(),
        p.magUr(), 
        p.rhoc(), 
        p.rhod(), 
        p.muc(), 
        p.phaseFraction(), 
        p.kth()
    );

    //- Calculate the drag Kd
    Kd_[p.kth()][p.cellI()] = 
        p.phaseFraction()*0.75*p.muc()/sqr(p.d())
       *dragModelPtr_->CdRe(p.kth(), p.cellI());


    //- Calculate the lift force
    liftF_[p.kth()][p.cellI()] = 
        liftModelPtr_->F
        (
            p.cellI(), 
            p.rhod(), 
            p.rhoc(), 
            p.d(), 
            p.phaseFraction(), 
            Re_[p.kth()][p.cellI()],
            p.Uc(),
            p.Ud(),
            p.curlUc()
        ); 

    //- Wall lubrication force
    wallLubF_[p.kth()][p.cellI()] = 
        wallLubModelPtr_->F
        (
            p.cellI(), 
            p.rhod(), 
            p.rhoc(), 
            p.d(), 
            p.phaseFraction(), 
            p.alpha(), 
            Re_[p.kth()][p.cellI()], 
            p.Ur()
        ); 

    //- Turbulent dispersion force
    turbulentDisF_[p.kth()][p.cellI()] =        
        turbulentDisModelPtr_->F
        (
            p.cellI(), 
            p.k(),
            p.rhoc(),
            p.gradPhaseFraction(),
            p.phaseFraction(),
            p.d(),
            p.kth(),
            p.nut(),
            p.magUr()
        ); 
}
