/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "Tomiyama.H"
#include "addToRunTimeSelectionTable.H"


namespace Foam
{
    namespace liftModels
    {
        defineTypeNameAndDebug(Tomiyama, 0);
    
        addToRunTimeSelectionTable
        (
            lift,
            Tomiyama,
            dictionary
        );
    }
}


Foam::vector Foam::liftModels::Tomiyama::F
(
    const label& cellI,
    const scalar& rhod,
    const scalar& rhoc,
    const scalar& diameter,
    const scalar& phaseF,
    const scalar& Re,
    const vector Uc,
    const vector Ud,
    const vector curlUc
)
{
    scalar Eo
    (
        mag(rhod - rhoc)*9.81*sqr(diameter)/surfaceT_ 
    );

    scalar dh
    (
        diameter*cbrt(1.0 + 0.163*pow(Eo, 0.757))
        //diameter*cbrt(phaseF)
    );

    scalar EoH
    (
        mag(rhod - rhoc)*9.81*sqr(dh)/surfaceT_
    );

    scalar f
    (
        0.0010422*pow3(EoH) - 0.0159*sqr(EoH) - 0.0204*EoH + 0.474
    );

    //- Cos
    scalar limiter
    (
        0.5*
        (
        1.0
          - cos
            (
                constant::mathematical::pi
               *min(wallDis_.yWall()[cellI]/(1.5*diameter), scalar(1))
            )
        )
    );

    scalar Cl
    (
        neg(EoH - scalar(4))*min(0.288*tanh(0.121*Re), f)
      + pos(EoH - scalar(4))*neg(EoH - scalar(10.7))*f
      + pos(EoH - scalar(10.7))*(-0.288)
    );

    return phaseF*limiter*Cl*rhoc/rhod*((Uc - Ud) ^ curlUc);
}


Foam::liftModels::Tomiyama::Tomiyama
(
    const dictionary& liftModels,
    const fvMesh& mesh
)
:
    lift(liftModels, mesh),
    surfaceT_(readScalar(liftModels_.lookup("surfaceTension")))
{}
