/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "Frank.H"
#include "addToRunTimeSelectionTable.H"


namespace Foam
{
    namespace wallLubricationModels
    {
        defineTypeNameAndDebug(Frank, 0);
    
        addToRunTimeSelectionTable
        (
            wallLubrication,
            Frank,
            dictionary
        );
    }
}


Foam::vector Foam::wallLubricationModels::Frank::F
(
    const label& cellI,
    const scalar& rhod,
    const scalar& rhoc,
    const scalar& diameter,
    const scalar& phaseF,
    const scalar& alpha,
    const scalar& Re,
    const vector& Ur	
)
{
    const vector n(-nWall()[cellI]);
    const scalar y(yWall()[cellI]);

    scalar Eo(mag(rhoc - rhod)*9.81*sqr(diameter)/surfaceT_);

    scalar yTilde(y/(Cwc_*diameter));

    return 
        (
            pos0(Eo - 1.0)*neg(Eo - 5.0)*exp(-0.933*Eo + 0.179)
          + pos0(Eo - 5.0)*neg(Eo - 33.0)*(0.00599*Eo - 0.0187)
          + pos0(Eo - 33.0)*0.179
        )
       *max
        (
            0,
            (1.0 - yTilde)/(Cwd_*y*pow(yTilde, p_ - 1.0))
        )
       *rhoc*magSqr(Ur - (Ur & n)*n)*n;
}


Foam::wallLubricationModels::Frank::Frank
(
    const dictionary& wallLubricationModels,
    const fvMesh& mesh
)
:
    wallLubrication(wallLubricationModels, mesh),
    surfaceT_(readScalar(wallLubricationModels_.lookup("surfaceTension"))),
    Cwd_(readScalar(wallLubricationModels_.lookup("Cwd"))),
    Cwc_(readScalar(wallLubricationModels_.lookup("Cwc"))),
    p_(readScalar(wallLubricationModels_.lookup("p")))
{}
