/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "Hosokawa.H"
#include "addToRunTimeSelectionTable.H"


namespace Foam
{
    namespace wallLubricationModels
    {
        defineTypeNameAndDebug(Hosokawa, 0);
    
        addToRunTimeSelectionTable
        (
            wallLubrication,
            Hosokawa,
            dictionary
        );
    }
}


Foam::vector Foam::wallLubricationModels::Hosokawa::F
(
    const label& cellI,
    const scalar& rhod,
    const scalar& rhoc,
    const scalar& diameter,
    const scalar& phaseF,
    const scalar& alpha,
    const scalar& Re,
    const vector& Ur	
)
{
    const vector n(-nWall()[cellI]);
    const scalar y(-yWall()[cellI]);

    scalar Eo(mag(rhoc - rhod)*9.81*sqr(diameter)/surfaceT_);

    if (alpha > 1e-5)
    {
        return max(7.0/pow(Re, 1.9), 0.0217*Eo)
          *0.5*phaseF
          *diameter
          *(
               1.0/sqr(y) 
             - 1.0/sqr(D_ - y)
           )
          *rhoc*magSqr(-Ur - ((-Ur) & n)*n)*n;
    }
    else
    {
        return Zero;
    }
}


Foam::wallLubricationModels::Hosokawa::Hosokawa
(
    const dictionary& wallLubricationModels,
    const fvMesh& mesh
)
:
    wallLubrication(wallLubricationModels, mesh),
    surfaceT_(readScalar(wallLubricationModels_.lookup("surfaceTension"))),
    D_(readScalar(wallLubricationModels_.lookup("pipeDiameter")))
{}
