/*******************************************************************************
@GPBEFoam:           3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:            Dongyue Li (HZDR & Dyfluid, Programmer)
                     Daniele Marchisio (Politecnico di Torino, Co-PI)
                     Christian Hasse (TU Darmstadt, Co-PI)
                     Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:  	     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                     oneWayGPBEFoam: open-source Eulerian-QBMM solvers for 
                     multiphase flows. Computer Physics Communication, Submitted

@History:            ver 1.0, 07/31/18 

@Info:	             The released Eulerian-QBMM library is based on OpenFOAM-5.x 
                     oneWayGPBEFoam can be employed to simulate one-way coupled 
                     problems. twoWayGPBEFoam can be employed to simulate
                     two-way coupled problems.
*******************************************************************************/

#include "wallLubrication.H"
#include "zeroGradientFvPatchFields.H"
#include "wallDist.H"


namespace Foam
{
    namespace wallLubricationModels
    {
        defineTypeNameAndDebug(wallLubrication, 0);
        defineRunTimeSelectionTable(wallLubrication, dictionary);
    }
}


Foam::wallLubricationModels::wallLubrication::wallLubrication
(
    const dictionary& forceModels,
    const fvMesh& mesh
)
:
    wallLubricationModels_(forceModels.subDict("wallLubricationModels")),
    mesh_(mesh),
    C_
    (
        IOobject
        (
            "C_",
            mesh_.time().timeName(),
            mesh_,
            IOobject::NO_READ,
            IOobject::NO_WRITE
        ),
        mesh_,
        0.0
    )
{}


const Foam::volScalarField& Foam::wallLubricationModels::wallLubrication::yWall() const
{
    return wallDist::New(mesh_).y();
}


const Foam::volVectorField& Foam::wallLubricationModels::wallLubrication::nWall() const
{
    return wallDist::New(mesh_).n();
}


Foam::autoPtr<Foam::wallLubricationModels::wallLubrication> Foam::wallLubricationModels::wallLubrication::New
(
    const dictionary& forceModels,
    const fvMesh& mesh
)
{
    const word modelType(forceModels.subDict("wallLubricationModels").lookup("wallLubricationModel"));

    Info << "Selecting wallLubrication model " << modelType 
        << endl;

    dictionaryConstructorTable::iterator cstrIter =
        dictionaryConstructorTablePtr_->find(modelType);

    if (cstrIter == dictionaryConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown wallLubrication type "
            << modelType << nl << nl
            << "Valid wallLubrications are : " << endl
            << dictionaryConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return autoPtr<wallLubrication>
        (cstrIter()(forceModels, mesh));
}
