/*******************************************************************************
@GPBEFoam:      3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:       Dongyue Li (HZDR & Dyfluid, Programmer)
                Daniele Marchisio (Politecnico di Torino, Co-PI)
                Christian Hasse (TU Darmstadt, Co-PI)
                Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                oneWayGPBEFoam: open-source Eulerian-QBMM solvers for multiphase 
                flows. Computer Physics Communication, Submitted

@History:       ver 1.0, 07/31/18 

@Info:          The released Eulerian-QBMM library is based on OpenFOAM-5.x. 
                oneWayGPBEFoam can be employed to simulate one-way coupled 
                problems. twoWayGPBEFoam can be employed to simulate two-way 
                coupled problems.
*******************************************************************************/

#include "LaakkonenBreakage.H"
#include "addToRunTimeSelectionTable.H"


namespace Foam
{
    defineTypeNameAndDebug(LaakkonenBreakage, 0);

    addToRunTimeSelectionTable
    (
        breakageModel,
        LaakkonenBreakage,
        dictionary
    );
}


void Foam::LaakkonenBreakage::updateBreakage
(
    PtrList<volScalarField>& w,
    PtrList<volScalarField>& a
)
{
    const volScalarField& rhoc =  
        a[0].db().lookupObject<volScalarField>("rho.water");

    const volScalarField& epsilon =  
        a[0].db().lookupObject<volScalarField>("epsilon.water");

    const volScalarField& muc =  
        a[0].db().lookupObject<volScalarField>("mu.water");

    for (int n = 0; n < 3; n++)
    {
        g_[n].ref() = 
            C1_*cbrt(epsilon)
           *erfc
            (
                sqrt
                (
                    C2_*sigma_
                   /(
                        rhoc*pow(a[n], 5.0/3.0)*pow(epsilon, 2.0/3.0)
                    )
                  + C3_*muc
                   /(
                        sqrt(rhoc*rhod_)*cbrt(epsilon)*pow(a[n], 4.0/3.0)
                    )
                )
            );
    }
}


Foam::LaakkonenBreakage::LaakkonenBreakage
(
    const dictionary& breakageModelProperties,
    const fvMesh& mesh
)
:
    breakageModel(breakageModelProperties, mesh),
    C1_
    (
        "C1",
        dimensionSet(0, -2.0/3.0, 0, 0, 0),
        readScalar(breakageModelProperties.lookup("C1"))
    ),
    C2_("C2", dimless, readScalar(breakageModelProperties.lookup("C2"))),
    C3_("C3", dimless, readScalar(breakageModelProperties.lookup("C3")))
{}
