/*******************************************************************************
@GPBEFoam:      3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:       Dongyue Li (HZDR & Dyfluid, Programmer)
                Daniele Marchisio (Politecnico di Torino, Co-PI)
                Christian Hasse (TU Darmstadt, Co-PI)
                Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                oneWayGPBEFoam: open-source Eulerian-QBMM solvers for multiphase 
                flows. Computer Physics Communication, Submitted

@History:       ver 1.0, 07/31/18 

@Info:          The released Eulerian-QBMM library is based on OpenFOAM-5.x. 
                oneWayGPBEFoam can be employed to simulate one-way coupled 
                problems. twoWayGPBEFoam can be employed to simulate two-way 
                coupled problems.
*******************************************************************************/

#include "breakageModel.H"
#include "zeroGradientFvPatchFields.H"

namespace Foam
{
    defineTypeNameAndDebug(breakageModel, 0);
    defineRunTimeSelectionTable(breakageModel, dictionary);
}


Foam::breakageModel::breakageModel
(
    const dictionary& breakageModelProperties,
    const fvMesh& mesh
)
:
    mesh_(mesh),
    g_(3),
    sigma_
    (
        "sigma",
        dimensionSet(1,0,-2,0,0),
        0.072
    ),
    rhod_
    (
        "rhod",
        dimensionSet(1,-3,0,0,0),
        1.0
    )
{
    forAll(g_, kth)
    {
        g_.set
        (
            kth,
            new tmp<volScalarField>
            (
                new volScalarField
                (
                    IOobject
                    (
                        "g" + name(kth), 
                        mesh_.time().timeName(),
                        mesh_,
                        IOobject::NO_READ,
                        IOobject::AUTO_WRITE
                    ),
                    mesh_,
                    dimensionedScalar
                    (
                        "g" + name(kth), 
                        dimensionSet(0,0,-1,0,0,0,0), 
                        0.0
                    ),
                    zeroGradientFvPatchField<scalar>::typeName
                )
            )
        );
    }
}


Foam::autoPtr<Foam::breakageModel> Foam::breakageModel::New
(
    const dictionary& breakageModelProperties,
    const fvMesh& mesh
)
{
    Info<< "Foam::breakageModel::New FUNCTION" << endl;

    const word modelType(breakageModelProperties.lookup("breakageModel"));

    Info<< "Selecting breakageModel model is " << modelType << endl;

    dictionaryConstructorTable::iterator cstrIter =
        dictionaryConstructorTablePtr_->find(modelType);


    if (cstrIter == dictionaryConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown breakageModelModel type "
            << modelType << nl << nl
            << "Valid breakageModel are : " << endl
            << dictionaryConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return autoPtr<breakageModel>
        (cstrIter()(breakageModelProperties, mesh));
}
