/*******************************************************************************
@GPBEFoam:      3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:       Dongyue Li (HZDR & Dyfluid, Programmer)
                Daniele Marchisio (Politecnico di Torino, Co-PI)
                Christian Hasse (TU Darmstadt, Co-PI)
                Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                oneWayGPBEFoam: open-source Eulerian-QBMM solvers for multiphase 
                flows. Computer Physics Communication, Submitted

@History:       ver 1.0, 07/31/18 

@Info:          The released Eulerian-QBMM library is based on OpenFOAM-5.x. 
                oneWayGPBEFoam can be employed to simulate one-way coupled 
                problems. twoWayGPBEFoam can be employed to simulate two-way 
                coupled problems.
*******************************************************************************/

#include "PrinceBlanch.H"
#include "addToRunTimeSelectionTable.H"
#include "uniformDimensionedFields.H"


namespace Foam
{
    defineTypeNameAndDebug(PrinceBlanch, 0);

    addToRunTimeSelectionTable
    (
        coalescenceModel,
        PrinceBlanch,
        dictionary
    );
}


const Foam::tmp<Foam::volScalarField>& Foam::PrinceBlanch::agg
(
    const PtrList<volScalarField>& a,
    const int i,
    const int j
)
{    
    const volScalarField& rhoc =  
        a[0].db().lookupObject<volScalarField>("rho.water");

    const volScalarField& epsilon =  
        a[0].db().lookupObject<volScalarField>("epsilon.water");

    const uniformDimensionedVectorField& g =
        a[0].db().lookupObject<uniformDimensionedVectorField>("g");

    const volScalarField rij(1.0/(1.0/a[i] + 1.0/a[j]));

    const volScalarField collisionEfficiency
    (
        exp
        (
          - sqrt
            (
                pow3(rij)*rhoc/(16.0*sigma_)
            )
           *log(h0_/hf_)
           *cbrt(epsilon)/pow(rij, 2.0/3.0)
        )
    );

    if (turbulentCollision_)
    {
        collisionRate_.ref() =
            C1_*M_PI*cbrt(epsilon)*sqr(a[i] + a[j])
           *sqrt(pow(a[i], 2.0/3.0) + pow(a[j], 2.0/3.0))
           *collisionEfficiency;
    }

    if (buoyantCollision_)
    {
        const volScalarField Sij(M_PI/4.0*sqr(a[i] + a[j]));

        collisionRate_.ref() =
            (
                Sij
               *mag
                (
                    sqrt
                    (
                        2.14*sigma_
                       /(rhoc*a[i]) + 0.505*mag(g)*a[i]
                    )
                  - sqrt
                    (
                        2.14*sigma_
                       /(rhoc*a[j]) + 0.505*mag(g)*a[j]
                    )
                )
            )
           *collisionEfficiency;
    }

    return collisionRate_;
}


Foam::PrinceBlanch::PrinceBlanch
(
    const dictionary& coalescenceModelProperties,
    const fvMesh& mesh
)
:
    coalescenceModel(coalescenceModelProperties, mesh),
    h0_
    (
        "h0",
        dimLength,
        readScalar(coalescenceModelProperties.lookup("h0"))
    ),
    hf_
    (
        "hf", 
        dimLength,
        readScalar(coalescenceModelProperties.lookup("hf"))
    ),
    C1_
    (
        "C1", 
        dimless,
        readScalar(coalescenceModelProperties.lookup("C1"))
    ),
    turbulentCollision_
    (
        coalescenceModelProperties.lookup("turbulentCollision")
    ),
    buoyantCollision_
    (
        coalescenceModelProperties.lookup("buoyantCollision")
    )
{}


