/*******************************************************************************
@GPBEFoam:      3D fully coupled Eulerian-QBMM library for multiphase flows

@Authors:       Dongyue Li (HZDR & Dyfluid, Programmer)
                Daniele Marchisio (Politecnico di Torino, Co-PI)
                Christian Hasse (TU Darmstadt, Co-PI)
                Dirk Lucas (Helmholtz-Zentrum Dresden-Rossendorf, Investor)

@Reference:     D. Li, D. Marchisio, C. Hasse, D. Lucas, twoWayGPBEFoam and 
                oneWayGPBEFoam: open-source Eulerian-QBMM solvers for multiphase 
                flows. Computer Physics Communication, Submitted

@History:       ver 1.0, 07/31/18 

@Info:          The released Eulerian-QBMM library is based on OpenFOAM-5.x. 
                oneWayGPBEFoam can be employed to simulate one-way coupled 
                problems. twoWayGPBEFoam can be employed to simulate two-way 
                coupled problems.
*******************************************************************************/

#include "coalescenceModel.H"
#include "zeroGradientFvPatchFields.H"

namespace Foam
{
    defineTypeNameAndDebug(coalescenceModel, 0);
    defineRunTimeSelectionTable(coalescenceModel, dictionary);
}


Foam::coalescenceModel::coalescenceModel
(
    const dictionary& coalescenceModelProperties,
    const fvMesh& mesh
)
:
    mesh_(mesh),
    sigma_
    (
        "sigma",
        dimensionSet(1,0,-2,0,0),
        0.072
    ),
    collisionRate_
    (
        new volScalarField
        (
            IOobject
            (
                "collisionRate",
                mesh_.time().timeName(),
                mesh_,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            mesh_,
            dimensionedScalar
            (
                "collisionRate",
                dimensionSet(0,3,-1,0,0,0,0), 
                0.0
            ),
            zeroGradientFvPatchField<scalar>::typeName
        )
    )
{}


Foam::autoPtr<Foam::coalescenceModel> Foam::coalescenceModel::New
(
    const dictionary& coalescenceModelProperties,
    const fvMesh& mesh
)
{
    Info<< "Foam::coalescenceModel::New FUNCTION" << endl;

    const word modelType(coalescenceModelProperties.lookup("coalescenceModel"));

    Info<< "Selecting coalescenceModel model is " << modelType << endl;

    dictionaryConstructorTable::iterator cstrIter =
        dictionaryConstructorTablePtr_->find(modelType);


    if (cstrIter == dictionaryConstructorTablePtr_->end())
    {
        FatalErrorInFunction
            << "Unknown coalescenceModelModel type "
            << modelType << nl << nl
            << "Valid coalescenceModel are : " << endl
            << dictionaryConstructorTablePtr_->sortedToc()
            << exit(FatalError);
    }

    return autoPtr<coalescenceModel>
        (cstrIter()(coalescenceModelProperties, mesh));
}















